@@ -1,7 +1,7 @@
 /*
  * wee-list.c - sorted lists
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,6 +48,7 @@ weelist_new ()
     if (new_weelist)
     {
         new_weelist->items = NULL;
+        new_weelist->last_item = NULL;
         new_weelist->size = 0;
     }
     return new_weelist;
@@ -434,15 +435,22 @@ void
 weelist_print_log (struct t_weelist *weelist, const char *name)
 {
     struct t_weelist_item *ptr_item;
+    int i;
 
     log_printf ("[weelist %s (addr:0x%lx)]", name, weelist);
+    log_printf ("  items. . . . . . . . . : 0x%lx", weelist->items);
+    log_printf ("  last_item. . . . . . . : 0x%lx", weelist->last_item);
+    log_printf ("  size . . . . . . . . . : %d", weelist->size);
 
+    i = 0;
     for (ptr_item = weelist->items; ptr_item;
          ptr_item = ptr_item->next_item)
     {
-        log_printf ("  data . . . . . . . . . : '%s'",  ptr_item->data);
-        log_printf ("  user_data. . . . . . . : 0x%lx", ptr_item->user_data);
-        log_printf ("  prev_item. . . . . . . : 0x%lx", ptr_item->prev_item);
-        log_printf ("  next_item. . . . . . . : 0x%lx", ptr_item->next_item);
+        log_printf ("  [item %d (addr:0x%lx)]", i, ptr_item);
+        log_printf ("    data . . . . . . . . : '%s'",  ptr_item->data);
+        log_printf ("    user_data. . . . . . : 0x%lx", ptr_item->user_data);
+        log_printf ("    prev_item. . . . . . : 0x%lx", ptr_item->prev_item);
+        log_printf ("    next_item. . . . . . : 0x%lx", ptr_item->next_item);
+        i++;
     }
 }
