@@ -17,7 +17,7 @@
  *
  * weechat.c - WeeChat main functions
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -32,7 +32,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,10 +60,12 @@
 #include "wee-eval.h"
 #include "wee-hdata.h"
 #include "wee-hook.h"
+#include "wee-list.h"
 #include "wee-log.h"
 #include "wee-network.h"
 #include "wee-proxy.h"
 #include "wee-secure.h"
+#include "wee-secure-config.h"
 #include "wee-string.h"
 #include "wee-upgrade.h"
 #include "wee-utf8.h"
@@ -79,6 +81,7 @@
 #include "../plugins/plugin-api.h"
 
 
+int weechat_headless = 0;              /* 1 if running headless (no GUI)    */
 int weechat_debug_core = 0;            /* debug level for core              */
 char *weechat_argv0 = NULL;            /* WeeChat binary file name (argv[0])*/
 int weechat_upgrading = 0;             /* =1 if WeeChat is upgrading        */
@@ -87,21 +90,24 @@ time_t weechat_first_start_time = 0;   /* start time (used by /uptime cmd)  */
 int weechat_upgrade_count = 0;         /* number of /upgrade done           */
 struct timeval weechat_current_start_timeval; /* start time used to display */
                                        /* duration of /upgrade              */
-int weechat_quit = 0;                  /* = 1 if quit request from user     */
+volatile sig_atomic_t weechat_quit = 0;   /* = 1 if quit request from user  */
 volatile sig_atomic_t weechat_quit_signal = 0; /* signal received,          */
                                        /* WeeChat must quit                 */
 char *weechat_home = NULL;             /* home dir. (default: ~/.weechat)   */
+int weechat_home_temp = 0;             /* 1 if using a temporary home       */
+int weechat_home_delete_on_exit = 0;   /* 1 if home is deleted on exit      */
 int weechat_locale_ok = 0;             /* is locale OK?                     */
 char *weechat_local_charset = NULL;    /* example: ISO-8859-1, UTF-8        */
 int weechat_server_cmd_line = 0;       /* at least 1 server on cmd line     */
-int weechat_auto_load_plugins = 1;     /* auto load plugins                 */
+char *weechat_force_plugin_autoload = NULL; /* force load of plugins        */
 int weechat_plugin_no_dlclose = 0;     /* remove calls to dlclose for libs  */
                                        /* (useful with valgrind)            */
 int weechat_no_gnutls = 0;             /* remove init/deinit of gnutls      */
                                        /* (useful with valgrind/electric-f.)*/
 int weechat_no_gcrypt = 0;             /* remove init/deinit of gcrypt      */
                                        /* (useful with valgrind)            */
-char *weechat_startup_commands = NULL; /* startup commands (-r flag)        */
+struct t_weelist *weechat_startup_commands = NULL; /* startup commands      */
+                                                   /* (option -r)           */
 
 
 /*
@@ -148,18 +154,40 @@ weechat_display_usage ()
           "(default: ~/.weechat)\n"
           "                           (environment variable WEECHAT_HOME is "
           "read if this option is not given)\n"
+          "  -t, --temp-dir           create a temporary WeeChat home"
+          "directory and delete it on exit\n"
+          "                           (incompatible with option \"-d\")\n"
           "  -h, --help               display this help\n"
           "  -l, --license            display WeeChat license\n"
           "  -p, --no-plugin          don't load any plugin at startup\n"
-          "  -r, --run-command <cmd>  run command(s) after startup\n"
-          "                           (many commands can be separated by "
-          "semicolons)\n"
+          "  -P, --plugins <plugins>  load only these plugins at startup\n"
+          "                           (see /help weechat.plugin.autoload)\n"
+          "  -r, --run-command <cmd>  run command(s) after startup;\n"
+          "                           many commands can be separated by "
+          "semicolons,\n"
+          "                           this option can be given multiple "
+          "times\n"
           "  -s, --no-script          don't load any script at startup\n"
           "      --upgrade            upgrade WeeChat using session files "
           "(see /help upgrade in WeeChat)\n"
           "  -v, --version            display WeeChat version\n"
           "  plugin:option            option for plugin (see man weechat)\n"));
-    string_fprintf(stdout, "\n");
+    string_fprintf (stdout, "\n");
+
+    /* extra options in headless mode */
+    if (weechat_headless)
+    {
+        string_fprintf (stdout, _("Extra options in headless mode:\n"));
+        string_fprintf (
+            stdout,
+            _("      --daemon             run WeeChat as a daemon (fork, "
+              "new process group, file descriptors closed);\n"));
+        string_fprintf (
+            stdout,
+            _("                           (by default in headless mode "
+              "WeeChat is blocking and does not run in background)\n"));
+        string_fprintf (stdout, "\n");
+    }
 }
 
 /*
@@ -173,11 +201,13 @@ weechat_parse_args (int argc, char *argv[])
 {
     int i;
 
-    weechat_argv0 = (argv && argv[0]) ? strdup (argv[0]) : NULL;
+    weechat_argv0 = (argv[0]) ? strdup (argv[0]) : NULL;
     weechat_upgrading = 0;
     weechat_home = NULL;
+    weechat_home_temp = 0;
+    weechat_home_delete_on_exit = 0;
     weechat_server_cmd_line = 0;
-    weechat_auto_load_plugins = 1;
+    weechat_force_plugin_autoload = NULL;
     weechat_plugin_no_dlclose = 0;
 
     for (i = 1; i < argc; i++)
@@ -189,8 +219,10 @@ weechat_parse_args (int argc, char *argv[])
             weechat_shutdown (EXIT_SUCCESS, 0);
         }
         else if ((strcmp (argv[i], "-d") == 0)
-            || (strcmp (argv[i], "--dir") == 0))
+                 || (strcmp (argv[i], "--dir") == 0))
         {
+            if (weechat_home_temp)
+                weechat_home_temp = 0;
             if (i + 1 < argc)
             {
                 if (weechat_home)
@@ -205,6 +237,16 @@ weechat_parse_args (int argc, char *argv[])
                 weechat_shutdown (EXIT_FAILURE, 0);
             }
         }
+        else if ((strcmp (argv[i], "-t") == 0)
+                 || (strcmp (argv[i], "--temp-dir") == 0))
+        {
+            if (weechat_home)
+            {
+                free (weechat_home);
+                weechat_home = NULL;
+            }
+            weechat_home_temp = 1;
+        }
         else if ((strcmp (argv[i], "-h") == 0)
                 || (strcmp (argv[i], "--help") == 0))
         {
@@ -253,16 +295,36 @@ weechat_parse_args (int argc, char *argv[])
         else if ((strcmp (argv[i], "-p") == 0)
                  || (strcmp (argv[i], "--no-plugin") == 0))
         {
-            weechat_auto_load_plugins = 0;
+            if (weechat_force_plugin_autoload)
+                free (weechat_force_plugin_autoload);
+            weechat_force_plugin_autoload = strdup ("!*");
+        }
+        else if ((strcmp (argv[i], "-P") == 0)
+                 || (strcmp (argv[i], "--plugins") == 0))
+        {
+            if (i + 1 < argc)
+            {
+                if (weechat_force_plugin_autoload)
+                    free (weechat_force_plugin_autoload);
+                weechat_force_plugin_autoload = strdup (argv[++i]);
+            }
+            else
+            {
+                string_fprintf (stderr,
+                                _("Error: missing argument for \"%s\" option\n"),
+                                argv[i]);
+                weechat_shutdown (EXIT_FAILURE, 0);
+            }
         }
         else if ((strcmp (argv[i], "-r") == 0)
                  || (strcmp (argv[i], "--run-command") == 0))
         {
             if (i + 1 < argc)
             {
-                if (weechat_startup_commands)
-                    free (weechat_startup_commands);
-                weechat_startup_commands = strdup (argv[++i]);
+                if (!weechat_startup_commands)
+                    weechat_startup_commands = weelist_new ();
+                weelist_add (weechat_startup_commands, argv[++i],
+                             WEECHAT_LIST_POS_END, NULL);
             }
             else
             {
@@ -340,9 +402,60 @@ weechat_set_home_path (char *home_path)
 void
 weechat_create_home_dir ()
 {
-    char *ptr_weechat_home, *config_weechat_home;
+    char *temp_dir, *temp_home_template, *ptr_weechat_home;
+    char *config_weechat_home;
+    int length, add_separator;
     struct stat statinfo;
 
+    /* temporary WeeChat home */
+    if (weechat_home_temp)
+    {
+        temp_dir = util_get_temp_dir ();
+        if (!temp_dir || !temp_dir[0])
+        {
+            string_fprintf (stderr,
+                            _("Error: not enough memory for home "
+                              "directory\n"));
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        length = strlen (temp_dir) + 32 + 1;
+        temp_home_template = malloc (length);
+        if (!temp_home_template)
+        {
+            free (temp_dir);
+            string_fprintf (stderr,
+                            _("Error: not enough memory for home "
+                              "directory\n"));
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        add_separator = (temp_dir[strlen (temp_dir) - 1] != DIR_SEPARATOR_CHAR);
+        snprintf (temp_home_template, length,
+                  "%s%sweechat_temp_XXXXXX",
+                  temp_dir,
+                  add_separator ? DIR_SEPARATOR : "");
+        free (temp_dir);
+        ptr_weechat_home = mkdtemp (temp_home_template);
+        if (!ptr_weechat_home)
+        {
+            string_fprintf (stderr,
+                            _("Error: unable to create a temporary "
+                              "home directory (using template: \"%s\")\n"),
+                            temp_home_template);
+            free (temp_home_template);
+            weechat_shutdown (EXIT_FAILURE, 0);
+            /* make C static analyzer happy (never executed) */
+            return;
+        }
+        weechat_home = strdup (ptr_weechat_home);
+        free (temp_home_template);
+        weechat_home_delete_on_exit = 1;
+        return;
+    }
+
     /*
      * weechat_home is not set yet: look for environment variable
      * "WEECHAT_HOME"
@@ -403,6 +516,14 @@ weechat_create_home_dir ()
 void
 weechat_startup_message ()
 {
+    if (weechat_headless)
+    {
+        string_fprintf (stdout,
+                        _("WeeChat is running in headless mode "
+                          "(Ctrl-C to quit)."));
+        string_fprintf (stdout, "\n");
+    }
+
     if (CONFIG_BOOLEAN(config_startup_display_logo))
     {
         gui_chat_printf (
@@ -586,29 +707,36 @@ weechat_shutdown (int return_code, int crash)
     network_end ();
     debug_end ();
 
+    if (!crash && weechat_home_delete_on_exit)
+    {
+        /* remove temporary home (only if not crashing) */
+        util_rmtree (weechat_home);
+    }
+
     if (weechat_argv0)
         free (weechat_argv0);
     if (weechat_home)
         free (weechat_home);
     if (weechat_local_charset)
         free (weechat_local_charset);
+    if (weechat_force_plugin_autoload)
+        free (weechat_force_plugin_autoload);
+    if (weechat_startup_commands)
+        weelist_free (weechat_startup_commands);
 
     if (crash)
-        abort();
+        abort ();
     else if (return_code >= 0)
         exit (return_code);
 }
 
 /*
- * Initializes WeeChat.
+ * Initializes gettext.
  */
 
 void
-weechat_init (int argc, char *argv[], void (*gui_init_cb)())
+weechat_init_gettext ()
 {
-    weechat_first_start_time = time (NULL); /* initialize start time        */
-    gettimeofday (&weechat_current_start_timeval, NULL);
-
     weechat_locale_ok = (setlocale (LC_ALL, "") != NULL);   /* init gettext */
 #ifdef ENABLE_NLS
     bindtextdomain (PACKAGE, LOCALEDIR);
@@ -622,6 +750,17 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_local_charset = strdup ("");
 #endif /* HAVE_LANGINFO_CODESET */
     utf8_init ();
+}
+
+/*
+ * Initializes WeeChat.
+ */
+
+void
+weechat_init (int argc, char *argv[], void (*gui_init_cb)())
+{
+    weechat_first_start_time = time (NULL); /* initialize start time        */
+    gettimeofday (&weechat_current_start_timeval, NULL);
 
     /* catch signals */
     util_catch_signal (SIGINT, SIG_IGN);           /* signal ignored        */
@@ -640,7 +779,9 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     completion_init ();                 /* add core completion hooks        */
     gui_key_init ();                    /* init keys                        */
     network_init_gcrypt ();             /* init gcrypt                      */
-    if (!secure_init ())                /* init secured data options (sec.*)*/
+    if (!secure_init ())                /* init secured data                */
+        weechat_shutdown (EXIT_FAILURE, 0);
+    if (!secure_config_init ())         /* init secured data options (sec.*)*/
         weechat_shutdown (EXIT_FAILURE, 0);
     if (!config_weechat_init ())        /* init WeeChat options (weechat.*) */
         weechat_shutdown (EXIT_FAILURE, 0);
@@ -648,7 +789,7 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_create_home_dir ();         /* create WeeChat home directory    */
     log_init ();                        /* init log file                    */
     plugin_api_init ();                 /* create some hooks (info,hdata,..)*/
-    secure_read ();                     /* read secured data options        */
+    secure_config_read ();              /* read secured data options        */
     config_weechat_read ();             /* read WeeChat options             */
     network_init_gnutls ();             /* init GnuTLS                      */
 
@@ -665,7 +806,7 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_term_check ();              /* warning about wrong $TERM        */
     weechat_locale_check ();            /* warning about wrong locale       */
     command_startup (0);                /* command executed before plugins  */
-    plugin_init (weechat_auto_load_plugins, /* init plugin interface(s)     */
+    plugin_init (weechat_force_plugin_autoload, /* init plugin interface(s) */
                  argc, argv);
     command_startup (1);                /* commands executed after plugins  */
     if (!weechat_upgrading)
@@ -685,14 +826,14 @@ weechat_end (void (*gui_end_cb)(int clean_exit))
     plugin_end ();                      /* end plugin interface(s)          */
     if (CONFIG_BOOLEAN(config_look_save_config_on_exit))
         (void) config_weechat_write (); /* save WeeChat config file         */
-    (void) secure_write ();             /* save secured data                */
+    (void) secure_config_write ();      /* save secured data                */
 
     if (gui_end_cb)
         (*gui_end_cb) (1);              /* shut down WeeChat GUI            */
 
     proxy_free_all ();                  /* free all proxies                 */
     config_weechat_free ();             /* free WeeChat options             */
-    secure_free ();                     /* free secured data options        */
+    secure_config_free ();              /* free secured data options        */
     config_file_free_all ();            /* free all configuration files     */
     gui_key_end ();                     /* remove all keys                  */
     unhook_all ();                      /* remove all hooks                 */
