@@ -1,7 +1,7 @@
 /*
  * gui-color.c - color functions (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -112,6 +112,37 @@ char *gui_color_ansi[16] =
 };
 
 
+/*
+ * Returns a color code from an option, which can be a color or a string.
+ *
+ * Returns NULL if the option has a wrong type.
+ */
+
+const char *
+gui_color_from_option (struct t_config_option *option)
+{
+    if (!option)
+        return NULL;
+
+    switch (option->type)
+    {
+        case CONFIG_OPTION_TYPE_COLOR:
+            if (option->min < 0)
+            {
+                return gui_color_get_custom (
+                    gui_color_get_name (CONFIG_COLOR(option)));
+            }
+            return GUI_COLOR(option->min);
+        case CONFIG_OPTION_TYPE_STRING:
+            return gui_color_get_custom (CONFIG_STRING(option));
+        default:
+            return NULL;
+    }
+
+    /* never executed */
+    return NULL;
+}
+
 /*
  * Searches for a color with configuration option name.
  *
@@ -123,21 +154,24 @@ gui_color_search_config (const char *color_name)
 {
     struct t_config_option *ptr_option;
 
-    if (color_name)
+    if (!color_name)
+        return NULL;
+
+    /* search in weechat.conf colors (example: "chat_delimiters") */
+    for (ptr_option = weechat_config_section_color->options;
+         ptr_option; ptr_option = ptr_option->next_option)
     {
-        for (ptr_option = weechat_config_section_color->options;
-             ptr_option; ptr_option = ptr_option->next_option)
-        {
-            if (string_strcasecmp (ptr_option->name, color_name) == 0)
-            {
-                if (ptr_option->min < 0)
-                {
-                    return gui_color_get_custom (
-                        gui_color_get_name (CONFIG_COLOR(ptr_option)));
-                }
-                return GUI_COLOR(ptr_option->min);
-            }
-        }
+        if (string_strcasecmp (ptr_option->name, color_name) == 0)
+            return gui_color_from_option (ptr_option);
+    }
+
+    /* search in any configuration file (example: "irc.color.message_quit") */
+    if (strchr (color_name, '.'))
+    {
+        config_file_search_with_string (color_name, NULL, NULL, &ptr_option,
+                                        NULL);
+        if (ptr_option)
+            return gui_color_from_option (ptr_option);
     }
 
     /* color not found */
@@ -206,7 +240,7 @@ const char *
 gui_color_get_custom (const char *color_name)
 {
     int fg, bg, fg_term, bg_term, term_color;
-    static char color[32][32];
+    static char color[32][96];
     static int index_color = 0;
     char color_fg[32], color_bg[32];
     char *pos_delim, *str_fg, *pos_bg, *error, *color_attr;
@@ -726,11 +760,10 @@ gui_color_decode (const char *string, const char *replacement)
 char *
 gui_color_decode_ansi_cb (void *data, const char *text)
 {
-    unsigned long keep_colors;
     char *text2, **items, *output, str_color[128];
-    int i, length, num_items, value;
+    int i, keep_colors, length, num_items, value;
 
-    keep_colors = (unsigned long)data;
+    keep_colors = (data) ? 1 : 0;;
 
     /* if we don't keep colors of if text is empty, just return empty string */
     if (!keep_colors || !text || !text[0])
@@ -967,8 +1000,8 @@ gui_color_emphasize (const char *string,
                      regex_t *regex)
 {
     regmatch_t regex_match;
-    char *result, *result2, *string_no_color, *pos;
-    const char *ptr_string, *ptr_no_color, *color_emphasis;
+    char *result, *result2, *string_no_color;
+    const char *ptr_string, *ptr_no_color, *color_emphasis, *pos;
     int rc, length_search, length_emphasis, length_result;
     int pos1, pos2, real_pos1, real_pos2, count_emphasis;
 
@@ -1114,13 +1147,13 @@ gui_color_emphasize (const char *string,
 void
 gui_color_free (struct t_gui_color *color)
 {
-    if (color)
-    {
-        if (color->string)
-            free (color->string);
+    if (!color)
+        return;
 
-        free (color);
-    }
+    if (color->string)
+        free (color->string);
+
+    free (color);
 }
 
 /*
@@ -1152,20 +1185,20 @@ gui_color_palette_alloc_structs ()
 {
     if (!gui_color_hash_palette_color)
     {
-        gui_color_hash_palette_color = hashtable_new (32,
-                                                      WEECHAT_HASHTABLE_STRING,
-                                                      WEECHAT_HASHTABLE_POINTER,
-                                                      NULL,
-                                                      NULL);
+        gui_color_hash_palette_color = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_POINTER,
+            NULL, NULL);
         gui_color_hash_palette_color->callback_free_value = &gui_color_palette_free_value_cb;
     }
     if (!gui_color_hash_palette_alias)
     {
-        gui_color_hash_palette_alias = hashtable_new (32,
-                                                      WEECHAT_HASHTABLE_STRING,
-                                                      WEECHAT_HASHTABLE_INTEGER,
-                                                      NULL,
-                                                      NULL);
+        gui_color_hash_palette_alias = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_INTEGER,
+            NULL, NULL);
     }
     if (!gui_color_list_with_alias)
     {
