@@ -1,7 +1,7 @@
 /*
  * gui-completion.c - word completion according to context (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 #include "../core/wee-arraylist.h"
 #include "../core/wee-completion.h"
 #include "../core/wee-config.h"
+#include "../core/wee-hashtable.h"
 #include "../core/wee-hdata.h"
 #include "../core/wee-hook.h"
 #include "../core/wee-list.h"
@@ -56,7 +57,8 @@ int gui_completion_freeze = 0;         /* 1 to freeze completions (do not   */
  */
 
 int
-gui_completion_word_compare_cb (void *data, struct t_arraylist *arraylist,
+gui_completion_word_compare_cb (void *data,
+                                struct t_arraylist *arraylist,
                                 void *pointer1, void *pointer2)
 {
     struct t_gui_completion_word *completion_word1, *completion_word2;
@@ -76,7 +78,8 @@ gui_completion_word_compare_cb (void *data, struct t_arraylist *arraylist,
  */
 
 void
-gui_completion_word_free_cb (void *data, struct t_arraylist *arraylist,
+gui_completion_word_free_cb (void *data,
+                             struct t_arraylist *arraylist,
                              void *pointer)
 {
     struct t_gui_completion_word *completion_word;
@@ -112,10 +115,12 @@ gui_completion_buffer_init (struct t_gui_completion *completion,
     completion->direction = 0;
     completion->add_space = 1;
     completion->force_partial_completion = 0;
+    completion->reverse_partial_completion = 0;
 
-    completion->list = arraylist_new (32, 1, 0,
-                                      &gui_completion_word_compare_cb, NULL,
-                                      &gui_completion_word_free_cb, NULL);
+    completion->list = arraylist_new (
+        32, 1, 0,
+        &gui_completion_word_compare_cb, NULL,
+        &gui_completion_word_free_cb, NULL);
 
     completion->word_found = NULL;
     completion->word_found_is_nick = 0;
@@ -197,7 +202,11 @@ gui_completion_free_data (struct t_gui_completion *completion)
 void
 gui_completion_free (struct t_gui_completion *completion)
 {
+    if (!completion)
+        return;
+
     gui_completion_free_data (completion);
+
     free (completion);
 }
 
@@ -349,19 +358,26 @@ int
 gui_completion_nickncmp (const char *base_word, const char *nick, int max)
 {
     char *base_word2, *nick2;
-    int return_cmp;
+    int case_sensitive, return_cmp;
+
+    case_sensitive = CONFIG_BOOLEAN(config_completion_nick_case_sensitive);
 
     if (!CONFIG_STRING(config_completion_nick_ignore_chars)
         || !CONFIG_STRING(config_completion_nick_ignore_chars)[0]
-        || !base_word || !nick || !base_word[0] || !nick[0]
+        || !base_word[0] || !nick[0]
         || gui_completion_nick_has_ignored_chars (base_word))
-        return string_strncasecmp (base_word, nick, max);
+    {
+        return (case_sensitive) ?
+            strncmp (base_word, nick, max) :
+            string_strncasecmp (base_word, nick, max);
+    }
 
     base_word2 = gui_completion_nick_strdup_ignore_chars (base_word);
     nick2 = gui_completion_nick_strdup_ignore_chars (nick);
 
-    return_cmp = string_strncasecmp (base_word2, nick2,
-                                     utf8_strlen (base_word2));
+    return_cmp = (case_sensitive) ?
+        strncmp (base_word2, nick2, utf8_strlen (base_word2)) :
+        string_strncasecmp (base_word2, nick2, utf8_strlen (base_word2));
 
     free (base_word2);
     free (nick2);
@@ -415,6 +431,7 @@ gui_completion_list_add (struct t_gui_completion *completion, const char *word,
                           CONFIG_STRING(config_completion_nick_completer));
                 completion_word->word = strdup (buffer);
                 arraylist_insert (completion->list, index, completion_word);
+                completion->add_space = 0;
             }
             else
             {
@@ -495,6 +512,12 @@ gui_completion_build_list_template (struct t_gui_completion *completion,
                                                                         pos_end - pos);
                                     if (custom_completion)
                                     {
+                                        if (hashtable_has_key (
+                                                config_hashtable_completion_partial_templates,
+                                                custom_completion))
+                                        {
+                                            completion->reverse_partial_completion = 1;
+                                        }
                                         gui_completion_custom (completion,
                                                                custom_completion,
                                                                plugin);
@@ -706,8 +729,7 @@ gui_completion_find_context (struct t_gui_completion *completion,
                              const char *data, int size, int pos)
 {
     int i, command_arg, pos_start, pos_end;
-    const char *ptr_command, *ptr_data;
-    char *prev_char;
+    const char *ptr_command, *ptr_data, *prev_char;
 
     /* look for context */
     gui_completion_free_data (completion);
@@ -913,7 +935,7 @@ int
 gui_completion_common_prefix_size (struct t_arraylist *list,
                                    const char *utf_char)
 {
-    char *ptr_first_item, *ptr_char, *next_char;
+    const char *ptr_first_item, *ptr_char, *next_char;
     struct t_gui_completion_word *ptr_completion_word;
     int i;
 
@@ -1060,6 +1082,9 @@ gui_completion_complete (struct t_gui_completion *completion)
             partial_completion = CONFIG_BOOLEAN(config_completion_partial_completion_other);
     }
 
+    if (completion->reverse_partial_completion)
+        partial_completion ^= 1;
+
     common_prefix_size = 0;
     if (partial_completion
         && completion->list && (completion->list->size > 0))
@@ -1158,7 +1183,10 @@ gui_completion_complete (struct t_gui_completion *completion)
 
                     /* alert user of partial completion */
                     if (CONFIG_BOOLEAN(config_completion_partial_completion_alert))
-                        printf ("\a");
+                    {
+                        fprintf (stderr, "\a");
+                        fflush (stderr);
+                    }
 
                     /*
                      * send "partial_completion" signal, to display possible
@@ -1172,6 +1200,8 @@ gui_completion_complete (struct t_gui_completion *completion)
                 }
 
                 arraylist_clear (completion->partial_list);
+                (void) hook_signal_send ("partial_completion",
+                                        WEECHAT_HOOK_SIGNAL_STRING, NULL);
 
                 return;
             }
@@ -1237,7 +1267,8 @@ gui_completion_auto (struct t_gui_completion *completion)
         || (completion->base_word[0] == '~'))
     {
         if (completion->list->size == 0)
-            completion_list_add_filename_cb (NULL, NULL, NULL, completion);
+            completion_list_add_filename_cb (NULL, NULL, NULL, NULL,
+                                             completion);
         gui_completion_complete (completion);
         return;
     }
@@ -1347,15 +1378,16 @@ gui_completion_get_string (struct t_gui_completion *completion,
  */
 
 struct t_hdata *
-gui_completion_hdata_completion_cb (void *data, const char *hdata_name)
+gui_completion_hdata_completion_cb (const void *pointer, void *data,
+                                    const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
-    hdata = hdata_new (NULL, hdata_name, NULL, NULL,
-                       0, 0, NULL, NULL);
+    hdata = hdata_new (NULL, hdata_name, NULL, NULL, 0, 0, NULL, NULL);
     if (hdata)
     {
         HDATA_VAR(struct t_gui_completion, buffer, POINTER, 0, NULL, "buffer");
@@ -1369,6 +1401,7 @@ gui_completion_hdata_completion_cb (void *data, const char *hdata_name)
         HDATA_VAR(struct t_gui_completion, direction, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_completion, add_space, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_completion, force_partial_completion, INTEGER, 0, NULL, NULL);
+        HDATA_VAR(struct t_gui_completion, reverse_partial_completion, INTEGER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_completion, list, POINTER, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_completion, word_found, STRING, 0, NULL, NULL);
         HDATA_VAR(struct t_gui_completion, word_found_is_nick, INTEGER, 0, NULL, NULL);
@@ -1380,29 +1413,6 @@ gui_completion_hdata_completion_cb (void *data, const char *hdata_name)
     return hdata;
 }
 
-/*
- * Returns hdata for completion word.
- */
-
-struct t_hdata *
-gui_completion_hdata_completion_word_cb (void *data, const char *hdata_name)
-{
-    struct t_hdata *hdata;
-
-    /* make C compiler happy */
-    (void) data;
-
-    hdata = hdata_new (NULL, hdata_name, "prev_item", "next_item",
-                       0, 0, NULL, NULL);
-    if (hdata)
-    {
-        HDATA_VAR(struct t_gui_completion_word, word, STRING, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_completion_word, nick_completion, CHAR, 0, NULL, NULL);
-        HDATA_VAR(struct t_gui_completion_word, count, INTEGER, 0, NULL, NULL);
-    }
-    return hdata;
-}
-
 /*
  * Prints list of completion words in WeeChat log file (usually for crash dump).
  */
@@ -1432,23 +1442,24 @@ void
 gui_completion_print_log (struct t_gui_completion *completion)
 {
     log_printf ("[completion (addr:0x%lx)]", completion);
-    log_printf ("  buffer. . . . . . . . . : 0x%lx", completion->buffer);
-    log_printf ("  context . . . . . . . . : %d",    completion->context);
-    log_printf ("  base_command. . . . . . : '%s'",  completion->base_command);
-    log_printf ("  base_command_arg_index. : %d",    completion->base_command_arg_index);
-    log_printf ("  base_word . . . . . . . : '%s'",  completion->base_word);
-    log_printf ("  base_word_pos . . . . . : %d",    completion->base_word_pos);
-    log_printf ("  position. . . . . . . . : %d",    completion->position);
-    log_printf ("  args. . . . . . . . . . : '%s'",  completion->args);
-    log_printf ("  direction . . . . . . . : %d",    completion->direction);
-    log_printf ("  add_space . . . . . . . : %d",    completion->add_space);
-    log_printf ("  force_partial_completion: %d",    completion->force_partial_completion);
-    log_printf ("  list. . . . . . . . . . : 0x%lx", completion->list);
-    log_printf ("  word_found. . . . . . . : '%s'",  completion->word_found);
-    log_printf ("  word_found_is_nick. . . : %d",    completion->word_found_is_nick);
-    log_printf ("  position_replace. . . . : %d",    completion->position_replace);
-    log_printf ("  diff_size . . . . . . . : %d",    completion->diff_size);
-    log_printf ("  diff_length . . . . . . : %d",    completion->diff_length);
+    log_printf ("  buffer. . . . . . . . . . : 0x%lx", completion->buffer);
+    log_printf ("  context . . . . . . . . . : %d",    completion->context);
+    log_printf ("  base_command. . . . . . . : '%s'",  completion->base_command);
+    log_printf ("  base_command_arg_index. . : %d",    completion->base_command_arg_index);
+    log_printf ("  base_word . . . . . . . . : '%s'",  completion->base_word);
+    log_printf ("  base_word_pos . . . . . . : %d",    completion->base_word_pos);
+    log_printf ("  position. . . . . . . . . : %d",    completion->position);
+    log_printf ("  args. . . . . . . . . . . : '%s'",  completion->args);
+    log_printf ("  direction . . . . . . . . : %d",    completion->direction);
+    log_printf ("  add_space . . . . . . . . : %d",    completion->add_space);
+    log_printf ("  force_partial_completion. : %d",    completion->force_partial_completion);
+    log_printf ("  reverse_partial_completion: %d",    completion->reverse_partial_completion);
+    log_printf ("  list. . . . . . . . . . . : 0x%lx", completion->list);
+    log_printf ("  word_found. . . . . . . . : '%s'",  completion->word_found);
+    log_printf ("  word_found_is_nick. . . . : %d",    completion->word_found_is_nick);
+    log_printf ("  position_replace. . . . . : %d",    completion->position_replace);
+    log_printf ("  diff_size . . . . . . . . : %d",    completion->diff_size);
+    log_printf ("  diff_length . . . . . . . : %d",    completion->diff_length);
     if (completion->list)
     {
         log_printf ("");
