@@ -1,7 +1,7 @@
 /*
  * gui-layout.c - layout functions (used by all GUI)
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -122,7 +122,7 @@ gui_layout_add (struct t_gui_layout *layout)
 
     layout->prev_layout = last_gui_layout;
     layout->next_layout = NULL;
-    if (gui_layouts)
+    if (last_gui_layout)
         last_gui_layout->next_layout = layout;
     else
         gui_layouts = layout;
@@ -234,7 +234,7 @@ gui_layout_buffer_add (struct t_gui_layout *layout,
 
         /* add layout buffer to list */
         new_layout_buffer->prev_layout = layout->last_layout_buffer;
-        if (layout->layout_buffers)
+        if (layout->last_layout_buffer)
             (layout->last_layout_buffer)->next_layout = new_layout_buffer;
         else
             layout->layout_buffers = new_layout_buffer;
@@ -883,11 +883,13 @@ gui_layout_remove_all ()
  */
 
 struct t_hdata *
-gui_layout_hdata_layout_buffer_cb (void *data, const char *hdata_name)
+gui_layout_hdata_layout_buffer_cb (const void *pointer, void *data,
+                                   const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_layout", "next_layout",
@@ -908,11 +910,13 @@ gui_layout_hdata_layout_buffer_cb (void *data, const char *hdata_name)
  */
 
 struct t_hdata *
-gui_layout_hdata_layout_window_cb (void *data, const char *hdata_name)
+gui_layout_hdata_layout_window_cb (const void *pointer, void *data,
+                                   const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, NULL, NULL, 0, 0, NULL, NULL);
@@ -935,11 +939,13 @@ gui_layout_hdata_layout_window_cb (void *data, const char *hdata_name)
  */
 
 struct t_hdata *
-gui_layout_hdata_layout_cb (void *data, const char *hdata_name)
+gui_layout_hdata_layout_cb (const void *pointer, void *data,
+                            const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_layout", "next_layout",
