@@ -1,7 +1,7 @@
 /*
  * weechat-aspell-info.c - info for aspell plugin
  *
- * Copyright (C) 2013-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2013-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -37,7 +37,7 @@ weechat_aspell_info_info_aspell_dict_cb (const void *pointer, void *data,
                                          const char *arguments)
 {
     int rc;
-    long unsigned int value;
+    unsigned long value;
     struct t_gui_buffer *buffer;
     const char *buffer_full_name;
 
