@@ -1,7 +1,7 @@
 /*
  * irc-notify.c - notify lists for IRC plugin
  *
- * Copyright (C) 2010-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -208,7 +208,7 @@ irc_notify_new (struct t_irc_server *server, const char *nick, int check_away)
 
         /* add notify to notify list on server */
         new_notify->prev_notify = server->last_notify;
-        if (server->notify_list)
+        if (server->last_notify)
             server->last_notify->next_notify = new_notify;
         else
             server->notify_list = new_notify;
@@ -434,6 +434,9 @@ void
 irc_notify_free (struct t_irc_server *server, struct t_irc_notify *notify,
                  int remove_monitor)
 {
+    if (!server || !notify)
+        return;
+
     (void) weechat_hook_signal_send ("irc_notify_removing",
                                      WEECHAT_HOOK_SIGNAL_POINTER, notify);
 
@@ -505,40 +508,40 @@ irc_notify_display (struct t_irc_server *server, struct t_gui_buffer *buffer,
     if ((notify->is_on_server < 0)
         || (!notify->is_on_server && !notify->away_message))
     {
-        weechat_printf (buffer,
-                        "  %s%s%s @ %s%s%s: %s%s",
-                        irc_nick_color_for_server_message (server, NULL,
-                                                           notify->nick),
-                        notify->nick,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_CHAT_SERVER,
-                        notify->server->name,
-                        IRC_COLOR_RESET,
-                        (notify->is_on_server < 0) ? "" : IRC_COLOR_MESSAGE_QUIT,
-                        (notify->is_on_server < 0) ?
-                        /* TRANSLATORS: "unknown" is the status for /notify when ison answer has not been received (check pending) */
-                        _("unknown") :
-                        _("offline"));
+        weechat_printf (
+            buffer,
+            "  %s%s%s @ %s%s%s: %s%s",
+            irc_nick_color_for_msg (server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            IRC_COLOR_CHAT_SERVER,
+            notify->server->name,
+            IRC_COLOR_RESET,
+            (notify->is_on_server < 0) ? "" : IRC_COLOR_MESSAGE_QUIT,
+            (notify->is_on_server < 0) ?
+            /* TRANSLATORS: "unknown" is the status for /notify when ison answer has not been received (check pending) */
+            _("unknown") :
+            _("offline"));
     }
     else
     {
-        weechat_printf (buffer,
-                        "  %s%s%s @ %s%s%s: %s%s %s%s%s%s%s%s",
-                        irc_nick_color_for_server_message (server, NULL,
-                                                           notify->nick),
-                        notify->nick,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_CHAT_SERVER,
-                        notify->server->name,
-                        IRC_COLOR_RESET,
-                        IRC_COLOR_MESSAGE_JOIN,
-                        _("online"),
-                        IRC_COLOR_RESET,
-                        (notify->away_message) ? " (" : "",
-                        (notify->away_message) ? _("away") : "",
-                        (notify->away_message) ? ": \"" : "",
-                        (notify->away_message) ? notify->away_message : "",
-                        (notify->away_message) ? "\")" : "");
+        weechat_printf (
+            buffer,
+            "  %s%s%s @ %s%s%s: %s%s %s%s%s%s%s%s",
+            irc_nick_color_for_msg (server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            IRC_COLOR_CHAT_SERVER,
+            notify->server->name,
+            IRC_COLOR_RESET,
+            IRC_COLOR_MESSAGE_JOIN,
+            _("online"),
+            IRC_COLOR_RESET,
+            (notify->away_message) ? " (" : "",
+            (notify->away_message) ? _("away") : "",
+            (notify->away_message) ? ": \"" : "",
+            (notify->away_message) ? notify->away_message : "",
+            (notify->away_message) ? "\")" : "");
     }
 }
 
@@ -672,29 +675,29 @@ irc_notify_set_is_on_server (struct t_irc_notify *notify, const char *host,
     if (notify->is_on_server == is_on_server)
         return;
 
-    weechat_printf_tags (notify->server->buffer,
-                         irc_notify_get_tags (irc_config_look_notify_tags_ison,
-                                              (is_on_server) ? "join" : "quit",
-                                              notify->nick),
-                         (notify->is_on_server < 0) ?
-                         ((is_on_server) ?
-                          _("%snotify: %s%s%s%s%s%s%s%s%s is connected") :
-                          _("%snotify: %s%s%s%s%s%s%s%s%s is offline")) :
-                         ((is_on_server) ?
-                          _("%snotify: %s%s%s%s%s%s%s%s%s has connected") :
-                          _("%snotify: %s%s%s%s%s%s%s%s%s has quit")),
-                         weechat_prefix ("network"),
-                         irc_nick_color_for_server_message (notify->server,
-                                                            NULL,
-                                                            notify->nick),
-                         notify->nick,
-                         (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
-                         (host && host[0]) ? " (" : "",
-                         (host && host[0]) ? IRC_COLOR_CHAT_HOST : "",
-                         (host && host[0]) ? host : "",
-                         (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
-                         (host && host[0]) ? ")" : "",
-                         (is_on_server) ? IRC_COLOR_MESSAGE_JOIN : IRC_COLOR_MESSAGE_QUIT);
+    weechat_printf_date_tags (
+        notify->server->buffer,
+        0,
+        irc_notify_get_tags (irc_config_look_notify_tags_ison,
+                             (is_on_server) ? "join" : "quit",
+                             notify->nick),
+        (notify->is_on_server < 0) ?
+        ((is_on_server) ?
+         _("%snotify: %s%s%s%s%s%s%s%s%s is connected") :
+         _("%snotify: %s%s%s%s%s%s%s%s%s is offline")) :
+        ((is_on_server) ?
+         _("%snotify: %s%s%s%s%s%s%s%s%s has connected") :
+         _("%snotify: %s%s%s%s%s%s%s%s%s has quit")),
+        weechat_prefix ("network"),
+        irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+        notify->nick,
+        (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
+        (host && host[0]) ? " (" : "",
+        (host && host[0]) ? IRC_COLOR_CHAT_HOST : "",
+        (host && host[0]) ? host : "",
+        (host && host[0]) ? IRC_COLOR_CHAT_DELIMITERS : "",
+        (host && host[0]) ? ")" : "",
+        (is_on_server) ? IRC_COLOR_MESSAGE_JOIN : IRC_COLOR_MESSAGE_QUIT);
     irc_notify_send_signal (notify, (is_on_server) ? "join" : "quit", NULL);
 
     notify->is_on_server = is_on_server;
@@ -720,49 +723,46 @@ irc_notify_set_away_message (struct t_irc_notify *notify,
 
     if (!notify->away_message && away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "away",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is now away: \"%s\""),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET,
-                             away_message);
+        weechat_printf_date_tags (
+            notify->server->buffer,
+            0,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "away", notify->nick),
+            _("%snotify: %s%s%s is now away: \"%s\""),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            away_message);
         irc_notify_send_signal (notify, "away", away_message);
     }
     else if (notify->away_message && !away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "back",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is back"),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET);
+        weechat_printf_date_tags (
+            notify->server->buffer,
+            0,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "back", notify->nick),
+            _("%snotify: %s%s%s is back"),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET);
         irc_notify_send_signal (notify, "back", NULL);
     }
     else if (notify->away_message && away_message)
     {
-        weechat_printf_tags (notify->server->buffer,
-                             irc_notify_get_tags (irc_config_look_notify_tags_whois,
-                                                  "still_away",
-                                                  notify->nick),
-                             _("%snotify: %s%s%s is still away: \"%s\""),
-                             weechat_prefix ("network"),
-                             irc_nick_color_for_server_message (notify->server,
-                                                                NULL,
-                                                                notify->nick),
-                             notify->nick,
-                             IRC_COLOR_RESET,
-                             away_message);
+        weechat_printf_date_tags (
+            notify->server->buffer,
+            0,
+            irc_notify_get_tags (
+                irc_config_look_notify_tags_whois, "still_away", notify->nick),
+            _("%snotify: %s%s%s is still away: \"%s\""),
+            weechat_prefix ("network"),
+            irc_nick_color_for_msg (notify->server, 1, NULL, notify->nick),
+            notify->nick,
+            IRC_COLOR_RESET,
+            away_message);
         irc_notify_send_signal (notify, "still_away", away_message);
     }
 
@@ -776,7 +776,7 @@ irc_notify_set_away_message (struct t_irc_notify *notify,
  */
 
 int
-irc_notify_hsignal_cb (void *data, const char *signal,
+irc_notify_hsignal_cb (const void *pointer, void *data, const char *signal,
                        struct t_hashtable *hashtable)
 {
     const char *error, *server, *pattern, *command, *output;
@@ -788,6 +788,7 @@ irc_notify_hsignal_cb (void *data, const char *signal,
     struct t_irc_notify *ptr_notify;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
 
@@ -842,7 +843,8 @@ irc_notify_hsignal_cb (void *data, const char *signal,
             for (i = 0; i < num_messages; i++)
             {
                 irc_message_parse (ptr_server, messages[i], NULL, NULL, NULL,
-                                   NULL, NULL, NULL, &arguments);
+                                   NULL, NULL, NULL, &arguments, NULL, NULL,
+                                   NULL, NULL, NULL);
                 if (arguments)
                 {
                     pos = strchr (arguments, ' ');
@@ -923,8 +925,9 @@ irc_notify_hsignal_cb (void *data, const char *signal,
             {
                 for (i = 0; i < num_messages; i++)
                 {
-                    irc_message_parse (ptr_server, messages[0], NULL, NULL,
-                                       NULL, NULL, &irc_cmd, NULL, &arguments);
+                    irc_message_parse (ptr_server, messages[i], NULL, NULL,
+                                       NULL, NULL, &irc_cmd, NULL, &arguments,
+                                       NULL, NULL, NULL, NULL, NULL);
                     if (irc_cmd && arguments)
                     {
                         if (strcmp (irc_cmd, "401") == 0)
@@ -967,7 +970,7 @@ irc_notify_hsignal_cb (void *data, const char *signal,
  */
 
 int
-irc_notify_timer_ison_cb (void *data, int remaining_calls)
+irc_notify_timer_ison_cb (const void *pointer, void *data, int remaining_calls)
 {
     char *message, hash_key[32];
     const char *str_message;
@@ -976,6 +979,7 @@ irc_notify_timer_ison_cb (void *data, int remaining_calls)
     struct t_hashtable *hashtable;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) remaining_calls;
 
@@ -1026,12 +1030,14 @@ irc_notify_timer_ison_cb (void *data, int remaining_calls)
  */
 
 int
-irc_notify_timer_whois_cb (void *data, int remaining_calls)
+irc_notify_timer_whois_cb (const void *pointer, void *data,
+                           int remaining_calls)
 {
     struct t_irc_server *ptr_server;
     struct t_irc_notify *ptr_notify, *ptr_next_notify;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) remaining_calls;
 
@@ -1072,11 +1078,13 @@ irc_notify_timer_whois_cb (void *data, int remaining_calls)
  */
 
 struct t_hdata *
-irc_notify_hdata_notify_cb (void *data, const char *hdata_name)
+irc_notify_hdata_notify_cb (const void *pointer, void *data,
+                            const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = weechat_hdata_new (hdata_name, "prev_notify", "next_notify",
@@ -1169,7 +1177,7 @@ irc_notify_hook_timer_ison ()
 
     irc_notify_timer_ison = weechat_hook_timer (
         60 * 1000 * weechat_config_integer (irc_config_network_notify_check_ison),
-        0, 0, &irc_notify_timer_ison_cb, NULL);
+        0, 0, &irc_notify_timer_ison_cb, NULL, NULL);
 }
 
 /*
@@ -1184,7 +1192,7 @@ irc_notify_hook_timer_whois ()
 
     irc_notify_timer_whois = weechat_hook_timer (
         60 * 1000 * weechat_config_integer (irc_config_network_notify_check_whois),
-        0, 0, &irc_notify_timer_whois_cb, NULL);
+        0, 0, &irc_notify_timer_whois_cb, NULL, NULL);
 }
 
 /*
@@ -1199,7 +1207,7 @@ irc_notify_init ()
 
     irc_notify_hsignal = weechat_hook_hsignal ("irc_redirection_notify_*",
                                                &irc_notify_hsignal_cb,
-                                               NULL);
+                                               NULL, NULL);
 }
 
 /*
