@@ -2,7 +2,7 @@
  * weechat-php.c - PHP plugin for WeeChat
  *
  * Copyright (C) 2006-2017 Adam Saponara <as@php.net>
- * Copyright (C) 2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2017-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,7 +17,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
+ *
+ * OpenSSL licensing:
+ *
+ *   Additional permission under GNU GPL version 3 section 7:
+ *
+ *   If you modify the Program, or any covered work, by linking or
+ *   combining it with the OpenSSL project's OpenSSL library (or a
+ *   modified version of that library), containing parts covered by the
+ *   terms of the OpenSSL or SSLeay licenses, the licensors of the Program
+ *   grant you additional permission to convey the resulting work.
+ *   Corresponding Source for a non-source form of such a combination
+ *   shall include the source code for the parts of OpenSSL used as well
+ *   as that of the covered work.
  */
 
 #include <sapi/embed/php_embed.h>
@@ -37,9 +50,23 @@ WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 WEECHAT_PLUGIN_PRIORITY(4000);
 
-struct t_weechat_plugin *weechat_php_plugin;
+struct t_weechat_plugin *weechat_php_plugin = NULL;
+
+struct t_plugin_script_data php_data;
+
+struct t_config_file *php_config_file = NULL;
+struct t_config_option *php_config_look_check_license = NULL;
+struct t_config_option *php_config_look_eval_keep_context = NULL;
+
+int php_quiet = 0;
+
+struct t_plugin_script *php_script_eval = NULL;
+int php_eval_mode = 0;
+int php_eval_send_input = 0;
+int php_eval_exec_commands = 0;
+struct t_gui_buffer *php_eval_buffer = NULL;
+char *php_eval_output = NULL;
 
-int php_quiet;
 struct t_plugin_script *php_scripts = NULL;
 struct t_plugin_script *last_php_script = NULL;
 struct t_plugin_script *php_current_script = NULL;
@@ -84,6 +111,7 @@ const zend_function_entry weechat_functions[] = {
     PHP_FE(weechat_string_has_highlight, NULL)
     PHP_FE(weechat_string_has_highlight_regex, NULL)
     PHP_FE(weechat_string_mask_to_regex, NULL)
+    PHP_FE(weechat_string_format_size, NULL)
     PHP_FE(weechat_string_remove_color, NULL)
     PHP_FE(weechat_string_is_command_char, NULL)
     PHP_FE(weechat_string_input_for_buffer, NULL)
@@ -161,6 +189,7 @@ const zend_function_entry weechat_functions[] = {
     PHP_FE(weechat_hook_process, NULL)
     PHP_FE(weechat_hook_process_hashtable, NULL)
     PHP_FE(weechat_hook_connect, NULL)
+    PHP_FE(weechat_hook_line, NULL)
     PHP_FE(weechat_hook_print, NULL)
     PHP_FE(weechat_hook_signal, NULL)
     PHP_FE(weechat_hook_signal_send, NULL)
@@ -271,7 +300,8 @@ const zend_function_entry weechat_functions[] = {
 
 PHP_MINIT_FUNCTION(weechat)
 {
-    (void)type;
+    /* make C compiler happy */
+    (void) type;
 
     /* Register integer constants */
     #define PHP_WEECHAT_CONSTANT(NAME) \
@@ -459,11 +489,11 @@ weechat_php_func_map_get (const char* func_name)
     return weechat_hashtable_get (weechat_php_func_map, func_name);
 }
 
-char *
+const char *
 weechat_php_func_map_add (zval *ofunc)
 {
     zval *func;
-    char *func_name;
+    const char *func_name;
 
     if (!weechat_php_func_map)
     {
@@ -541,8 +571,8 @@ weechat_php_exec (struct t_plugin_script *script, int ret_type,
 
     /* Invoke func */
     ret_value = NULL;
-    memset(&fci, 0, sizeof (zend_fcall_info));
-    memset(&fci_cache, 0, sizeof (zend_fcall_info_cache));
+    memset (&fci, 0, sizeof (zend_fcall_info));
+    memset (&fci_cache, 0, sizeof (zend_fcall_info_cache));
 
     zfunc = weechat_php_func_map_get (function);
     if (zfunc && zend_fcall_info_init (zfunc, 0, &fci, &fci_cache, NULL, NULL) == SUCCESS)
@@ -561,6 +591,13 @@ weechat_php_exec (struct t_plugin_script *script, int ret_type,
                 convert_to_string (&zretval);
                 ret_value = strdup ((char *)Z_STRVAL(zretval));
             }
+            else if (ret_type == WEECHAT_SCRIPT_EXEC_POINTER)
+            {
+                convert_to_string (&zretval);
+                ret_value = plugin_script_str2ptr (weechat_php_plugin,
+                                                   script->name, function,
+                                                   (char *)Z_STRVAL(zretval));
+            }
             else if (ret_type == WEECHAT_SCRIPT_EXEC_INT)
             {
                 convert_to_long (&zretval);
@@ -577,7 +614,15 @@ weechat_php_exec (struct t_plugin_script *script, int ret_type,
             }
             else
             {
-                WEECHAT_SCRIPT_MSG_WRONG_ARGS(PHP_CURRENT_SCRIPT_NAME, function);
+                if (ret_type != WEECHAT_SCRIPT_EXEC_IGNORE)
+                {
+                    weechat_printf (NULL,
+                                    weechat_gettext ("%s%s: function \"%s\" "
+                                                     "must return a valid "
+                                                     "value"),
+                                    weechat_prefix ("error"), PHP_PLUGIN_NAME,
+                                    function);
+                }
             }
         }
         else
@@ -589,6 +634,13 @@ weechat_php_exec (struct t_plugin_script *script, int ret_type,
     }
     zend_end_try ();
 
+    if ((ret_type != WEECHAT_SCRIPT_EXEC_IGNORE) && !ret_value)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: error in function \"%s\""),
+                        weechat_prefix ("error"), PHP_PLUGIN_NAME, function);
+    }
+
     /* Cleanup */
     if (params)
     {
@@ -608,16 +660,21 @@ weechat_php_exec (struct t_plugin_script *script, int ret_type,
 /*
  * Loads a PHP script.
  *
- * Returns:
- *   1: OK
- *   0: error
+ * If code is NULL, the content of filename is read and executed.
+ * If code is not NULL, it is executed (the file is not read).
+ *
+ * Returns pointer to new registered script, NULL if error.
  */
 
-int
-weechat_php_load (const char *filename)
+struct t_plugin_script *
+weechat_php_load (const char *filename, const char *code)
 {
     zend_file_handle file_handle;
 
+    /* make C compiler happy */
+    /* TODO: implement load of code in PHP */
+    (void) code;
+
     if ((weechat_php_plugin->debug >= 2) || !php_quiet)
     {
         weechat_printf (NULL,
@@ -647,7 +704,7 @@ weechat_php_load (const char *filename)
                         weechat_gettext ("%s%s: function \"register\" not "
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), PHP_PLUGIN_NAME, filename);
-        return 0;
+        return NULL;
     }
     php_current_script = php_registered_script;
 
@@ -661,7 +718,7 @@ weechat_php_load (const char *filename)
                                      WEECHAT_HOOK_SIGNAL_STRING,
                                      php_current_script->filename);
 
-    return 1;
+    return php_current_script;
 }
 
 /*
@@ -674,7 +731,7 @@ weechat_php_load_cb (void *data, const char *filename)
     /* make C compiler happy */
     (void) data;
 
-    weechat_php_load (filename);
+    weechat_php_load (filename, NULL);
 }
 
 /*
@@ -782,7 +839,7 @@ weechat_php_reload_name (const char *name)
                                 weechat_gettext ("%s: script \"%s\" unloaded"),
                                 PHP_PLUGIN_NAME, name);
             }
-            weechat_php_load (filename);
+            weechat_php_load (filename, NULL);
             free (filename);
         }
     }
@@ -794,6 +851,27 @@ weechat_php_reload_name (const char *name)
     }
 }
 
+/*
+ * Evaluates PHP source code.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+weechat_php_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
+                  int exec_commands, const char *code)
+{
+    /* TODO: implement PHP eval */
+    (void) buffer;
+    (void) send_to_buffer_as_input;
+    (void) exec_commands;
+    (void) code;
+
+    return 1;
+}
+
 /*
  * Callback for command "/php".
  */
@@ -803,12 +881,12 @@ weechat_php_command_cb (const void *pointer, void *data,
                         struct t_gui_buffer *buffer,
                         int argc, char **argv, char **argv_eol)
 {
-    char *ptr_name, *path_script;
+    char *ptr_name, *ptr_code, *path_script;
+    int i, send_to_buffer_as_input, exec_commands;
 
     /* make C compiler happy */
     (void) pointer;
     (void) data;
-    (void) buffer;
 
     if (argc == 1)
     {
@@ -840,6 +918,10 @@ weechat_php_command_cb (const void *pointer, void *data,
         {
             weechat_php_unload_all ();
         }
+        else if (weechat_strcasecmp (argv[1], "version") == 0)
+        {
+            plugin_script_display_interpreter (weechat_php_plugin, 0);
+        }
         else
             WEECHAT_COMMAND_ERROR;
     }
@@ -874,7 +956,8 @@ weechat_php_command_cb (const void *pointer, void *data,
                 /* load PHP script */
                 path_script = plugin_script_search_path (weechat_php_plugin,
                                                          ptr_name);
-                weechat_php_load ((path_script) ? path_script : ptr_name);
+                weechat_php_load ((path_script) ? path_script : ptr_name,
+                                  NULL);
                 if (path_script)
                     free (path_script);
             }
@@ -890,6 +973,44 @@ weechat_php_command_cb (const void *pointer, void *data,
             }
             php_quiet = 0;
         }
+        else if (weechat_strcasecmp (argv[1], "eval") == 0)
+        {
+            send_to_buffer_as_input = 0;
+            exec_commands = 0;
+            ptr_code = argv_eol[2];
+            for (i = 2; i < argc; i++)
+            {
+                if (argv[i][0] == '-')
+                {
+                    if (strcmp (argv[i], "-o") == 0)
+                    {
+                        if (i + 1 >= argc)
+                            WEECHAT_COMMAND_ERROR;
+                        send_to_buffer_as_input = 1;
+                        exec_commands = 0;
+                        ptr_code = argv_eol[i + 1];
+                    }
+                    else if (strcmp (argv[i], "-oc") == 0)
+                    {
+                        if (i + 1 >= argc)
+                            WEECHAT_COMMAND_ERROR;
+                        send_to_buffer_as_input = 1;
+                        exec_commands = 1;
+                        ptr_code = argv_eol[i + 1];
+                    }
+                }
+                else
+                    break;
+            }
+            if (!weechat_php_eval (buffer, send_to_buffer_as_input,
+                                   exec_commands, ptr_code))
+                WEECHAT_COMMAND_ERROR;
+            /* TODO: implement /php eval */
+            weechat_printf (NULL,
+                            _("%sCommand \"/%s eval\" is not yet implemented"),
+                            weechat_prefix ("error"),
+                            weechat_php_plugin->name);
+        }
         else
             WEECHAT_COMMAND_ERROR;
     }
@@ -935,6 +1056,26 @@ weechat_php_hdata_cb (const void *pointer, void *data,
                                        hdata_name);
 }
 
+/*
+ * Returns PHP info "php_eval".
+ */
+
+const char *
+weechat_php_info_eval_cb (const void *pointer, void *data,
+                          const char *info_name,
+                          const char *arguments)
+{
+    static const char *not_implemented = "not yet implemented";
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+    (void) arguments;
+
+    return not_implemented;
+}
+
 /*
  * Returns infolist with PHP scripts.
  */
@@ -985,31 +1126,6 @@ weechat_php_signal_debug_dump_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
-/*
- * Display infos about external libraries used.
- */
-
-int
-weechat_php_signal_debug_libs_cb (const void *pointer, void *data,
-                                  const char *signal,
-                                  const char *type_data, void *signal_data)
-{
-    /* make C compiler happy */
-    (void) pointer;
-    (void) data;
-    (void) signal;
-    (void) type_data;
-    (void) signal_data;
-
-#ifdef PHP_VERSION
-    weechat_printf (NULL, "  %s: %s", PHP_PLUGIN_NAME, PHP_VERSION);
-#else
-    weechat_printf (NULL, "  %s: (?)", PHP_PLUGIN_NAME);
-#endif /* PHP_VERSION */
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Timer for executing actions.
  */
@@ -1114,10 +1230,15 @@ php_weechat_ub_write (const char *str, size_t str_length)
 void
 php_weechat_sapi_error (int type, const char *format, ...)
 {
+    /* make C compiler happy */
     (void) type;
+
     weechat_va_format (format);
-    php_weechat_ub_write (vbuffer, strlen (vbuffer));
-    free (vbuffer);
+    if (vbuffer)
+    {
+        php_weechat_ub_write (vbuffer, strlen (vbuffer));
+        free (vbuffer);
+    }
 }
 
 #if PHP_VERSION_ID >= 70100
@@ -1125,6 +1246,7 @@ php_weechat_sapi_error (int type, const char *format, ...)
 void
 php_weechat_log_message (char *message, int syslog_type_int)
 {
+    /* make C compiler happy */
     (void) syslog_type_int;
 
     php_weechat_ub_write (message, strlen (message));
@@ -1145,18 +1267,33 @@ php_weechat_log_message (char *message)
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    struct t_plugin_script_init init;
-
     weechat_php_plugin = plugin;
 
-    init.callback_command = &weechat_php_command_cb;
-    init.callback_completion = &weechat_php_completion_cb;
-    init.callback_hdata = &weechat_php_hdata_cb;
-    init.callback_infolist = &weechat_php_infolist_cb;
-    init.callback_signal_debug_dump = &weechat_php_signal_debug_dump_cb;
-    init.callback_signal_debug_libs = &weechat_php_signal_debug_libs_cb;
-    init.callback_signal_script_action = &weechat_php_signal_script_action_cb;
-    init.callback_load_file = &weechat_php_load_cb;
+    /* set interpreter name and version */
+    weechat_hashtable_set (plugin->variables, "interpreter_name",
+                           plugin->name);
+#ifdef PHP_VERSION
+    weechat_hashtable_set (plugin->variables, "interpreter_version",
+                           PHP_VERSION);
+#else
+    weechat_hashtable_set (plugin->variables, "interpreter_version",
+                           "");
+#endif /* PHP_VERSION */
+
+    php_data.config_file = &php_config_file;
+    php_data.config_look_check_license = &php_config_look_check_license;
+    php_data.config_look_eval_keep_context = &php_config_look_eval_keep_context;
+    php_data.scripts = &php_scripts;
+    php_data.last_script = &last_php_script;
+    php_data.callback_command = &weechat_php_command_cb;
+    php_data.callback_completion = &weechat_php_completion_cb;
+    php_data.callback_hdata = &weechat_php_hdata_cb;
+    php_data.callback_info_eval = &weechat_php_info_eval_cb;
+    php_data.callback_infolist = &weechat_php_infolist_cb;
+    php_data.callback_signal_debug_dump = &weechat_php_signal_debug_dump_cb;
+    php_data.callback_signal_script_action = &weechat_php_signal_script_action_cb;
+    php_data.callback_load_file = &weechat_php_load_cb;
+    php_data.unload_all = &weechat_php_unload_all;
 
     php_embed_module.startup = php_weechat_startup;
     php_embed_module.ub_write = php_weechat_ub_write;
@@ -1169,7 +1306,7 @@ weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
     PG(report_zend_debug) = 0;  /* Turn off --enable-debug output */
 
     php_quiet = 1;
-    plugin_script_init (weechat_php_plugin, argc, argv, &init);
+    plugin_script_init (weechat_php_plugin, argc, argv, &php_data);
     php_quiet = 0;
 
     plugin_script_display_short_list (weechat_php_plugin,
@@ -1188,7 +1325,12 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
     /* unload all scripts */
     php_quiet = 1;
-    plugin_script_end (plugin, &php_scripts, &weechat_php_unload_all);
+    if (php_script_eval)
+    {
+        weechat_php_unload (php_script_eval);
+        php_script_eval = NULL;
+    }
+    plugin_script_end (plugin, &php_data);
     php_quiet = 0;
 
     if (weechat_php_func_map)
@@ -1206,6 +1348,9 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         free (php_action_remove_list);
     if (php_action_autoload_list)
         free (php_action_autoload_list);
+    /* weechat_string_dyn_free (php_buffer_output, 1); */
+    if (php_eval_output)
+        free (php_eval_output);
 
     return WEECHAT_RC_OK;
 }
