@@ -1,7 +1,7 @@
 /*
  * relay-weechat-msg.c - build binary messages for WeeChat protocol
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -226,7 +226,7 @@ relay_weechat_msg_add_pointer (struct t_relay_weechat_msg *msg, void *pointer)
     unsigned char length;
 
     snprintf (str_pointer, sizeof (str_pointer),
-              "%lx", (long unsigned int)pointer);
+              "%lx", (unsigned long)pointer);
     length = strlen (str_pointer);
     relay_weechat_msg_add_bytes (msg, &length, 1);
     relay_weechat_msg_add_bytes (msg, str_pointer, length);
@@ -557,7 +557,7 @@ relay_weechat_msg_add_hdata (struct t_relay_weechat_msg *msg,
     char *path_returned;
     const char *hdata_name, *array_size;
     void *pointer, **path_pointers;
-    long unsigned int value;
+    unsigned long value;
     int rc, num_keys, num_path, i, type, pos_count, count, rc_sscanf;
     uint32_t count32;
 
@@ -904,9 +904,9 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
                 relay_weechat_msg_add_pointer (msg, ptr_nick);
                 relay_weechat_msg_add_char (msg, 0); /* group */
                 relay_weechat_msg_add_char (msg,
-                                            (char)weechat_hdata_integer(ptr_hdata_nick,
-                                                                        ptr_nick,
-                                                                        "visible"));
+                                            (char)weechat_hdata_integer (ptr_hdata_nick,
+                                                                         ptr_nick,
+                                                                         "visible"));
                 relay_weechat_msg_add_int (msg, 0); /* level */
                 relay_weechat_msg_add_string (msg,
                                               weechat_hdata_string (ptr_hdata_nick,
@@ -932,9 +932,9 @@ relay_weechat_msg_add_nicklist_buffer (struct t_relay_weechat_msg *msg,
                 relay_weechat_msg_add_pointer (msg, ptr_group);
                 relay_weechat_msg_add_char (msg, 1); /* group */
                 relay_weechat_msg_add_char (msg,
-                                            (char)weechat_hdata_integer(ptr_hdata_group,
-                                                                        ptr_group,
-                                                                        "visible"));
+                                            (char)weechat_hdata_integer (ptr_hdata_group,
+                                                                         ptr_group,
+                                                                         "visible"));
                 relay_weechat_msg_add_int (msg,
                                            weechat_hdata_integer (ptr_hdata_group,
                                                                   ptr_group,
