@@ -1,7 +1,7 @@
 /*
  * relay-weechat-protocol.c - WeeChat protocol for relay to client
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -63,7 +63,7 @@ struct t_gui_buffer *
 relay_weechat_protocol_get_buffer (const char *arg)
 {
     struct t_gui_buffer *ptr_buffer;
-    long unsigned int value;
+    unsigned long value;
     int rc;
     struct t_hdata *ptr_hdata;
 
@@ -169,15 +169,16 @@ relay_weechat_protocol_is_sync (struct t_relay_client *ptr_client,
 
 RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
 {
-    char **options, *pos, *password;
-    int num_options, i, compression;
+    char **options, *pos, *password, *totp_secret, *info_totp_args;
+    const char *info_totp;
+    int i, compression, length;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(1);
 
-    options = weechat_string_split (argv_eol[0], ",", 0, 0, &num_options);
+    options = weechat_string_split_command (argv_eol[0], ',');
     if (options)
     {
-        for (i = 0; i < num_options; i++)
+        for (i = 0; options[i]; i++)
         {
             pos = strchr (options[i], '=');
             if (pos)
@@ -186,20 +187,41 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
                 pos++;
                 if (strcmp (options[i], "password") == 0)
                 {
-                    password = weechat_string_eval_expression (weechat_config_string (relay_config_network_password),
-                                                               NULL, NULL, NULL);
+                    password = weechat_string_eval_expression (
+                        weechat_config_string (relay_config_network_password),
+                        NULL, NULL, NULL);
                     if (password)
                     {
                         if (strcmp (password, pos) == 0)
-                        {
                             RELAY_WEECHAT_DATA(client, password_ok) = 1;
-                            weechat_hook_signal_send ("relay_client_auth_ok",
-                                                      WEECHAT_HOOK_SIGNAL_POINTER,
-                                                      client);
-                        }
                         free (password);
                     }
                 }
+                else if (strcmp (options[i], "totp") == 0)
+                {
+                    totp_secret = weechat_string_eval_expression (
+                        weechat_config_string (relay_config_network_totp_secret),
+                        NULL, NULL, NULL);
+                    if (totp_secret)
+                    {
+                        length = strlen (totp_secret) + strlen (pos) + 16 + 1;
+                        info_totp_args = malloc (length);
+                        if (info_totp_args)
+                        {
+                            /* validate the OTP received from the client */
+                            snprintf (info_totp_args, length,
+                                      "%s,%s,0,%d",
+                                      totp_secret,  /* the shared secret */
+                                      pos,          /* the OTP from client */
+                                      weechat_config_integer (relay_config_network_totp_window));
+                            info_totp = weechat_info_get ("totp_validate", info_totp_args);
+                            if (info_totp && (strcmp (info_totp, "1") == 0))
+                                RELAY_WEECHAT_DATA(client, totp_ok) = 1;
+                            free (info_totp_args);
+                        }
+                        free (totp_secret);
+                    }
+                }
                 else if (strcmp (options[i], "compression") == 0)
                 {
                     compression = relay_weechat_compression_search (pos);
@@ -208,7 +230,19 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(init)
                 }
             }
         }
-        weechat_string_free_split (options);
+        weechat_string_free_split_command (options);
+    }
+
+    if (RELAY_WEECHAT_DATA(client, password_ok)
+        && RELAY_WEECHAT_DATA(client, totp_ok))
+    {
+        weechat_hook_signal_send ("relay_client_auth_ok",
+                                  WEECHAT_HOOK_SIGNAL_POINTER,
+                                  client);
+    }
+    else
+    {
+        relay_client_set_status (client, RELAY_STATUS_AUTH_FAILED);
     }
 
     return WEECHAT_RC_OK;
@@ -231,11 +265,15 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(hdata)
     msg = relay_weechat_msg_new (id);
     if (msg)
     {
-        if (relay_weechat_msg_add_hdata (msg, argv[0],
-                                         (argc > 1) ? argv_eol[1] : NULL))
+        if (!relay_weechat_msg_add_hdata (msg, argv[0],
+                                          (argc > 1) ? argv_eol[1] : NULL))
         {
-            relay_weechat_msg_send (client, msg);
+            relay_weechat_msg_add_type (msg, RELAY_WEECHAT_MSG_OBJ_HDATA);
+            relay_weechat_msg_add_string (msg, NULL);  /* h-path */
+            relay_weechat_msg_add_string (msg, NULL);  /* keys */
+            relay_weechat_msg_add_int (msg, 0);  /* count */
         }
+        relay_weechat_msg_send (client, msg);
         relay_weechat_msg_free (msg);
     }
 
@@ -281,7 +319,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(info)
 RELAY_WEECHAT_PROTOCOL_CALLBACK(infolist)
 {
     struct t_relay_weechat_msg *msg;
-    long unsigned int value;
+    unsigned long value;
     char *args;
     int rc;
 
@@ -359,7 +397,8 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(nicklist)
  */
 
 int
-relay_weechat_protocol_input_timer_cb (void *data,
+relay_weechat_protocol_input_timer_cb (const void *pointer,
+                                       void *data,
                                        int remaining_calls)
 {
     char **timer_args;
@@ -367,9 +406,10 @@ relay_weechat_protocol_input_timer_cb (void *data,
     struct t_gui_buffer *ptr_buffer;
 
     /* make C compiler happy */
+    (void) data;
     (void) remaining_calls;
 
-    timer_args = (char **)data;
+    timer_args = (char **)pointer;
 
     if (!timer_args)
         return WEECHAT_RC_ERROR;
@@ -405,7 +445,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
     struct t_gui_buffer *ptr_buffer;
     char *pos, **timer_args;
 
-    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(2);
+    RELAY_WEECHAT_PROTOCOL_MIN_ARGS(1);
 
     ptr_buffer = relay_weechat_protocol_get_buffer (argv[0]);
     if (!ptr_buffer)
@@ -417,7 +457,7 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
                               "\"%s %s\""),
                             RELAY_PLUGIN_NAME,
                             command,
-                            argv_eol[0]);
+                            argv[0]);
         }
         return WEECHAT_RC_OK;
     }
@@ -438,7 +478,8 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
             timer_args[1] = strdup (pos + 1);
             weechat_hook_timer (1, 0, 1,
                                 &relay_weechat_protocol_input_timer_cb,
-                                timer_args);
+                                timer_args,
+                                NULL);
         }
     }
 
@@ -450,7 +491,8 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(input)
  */
 
 int
-relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
+relay_weechat_protocol_signal_buffer_cb (const void *pointer, void *data,
+                                         const char *signal,
                                          const char *type_data,
                                          void *signal_data)
 {
@@ -463,9 +505,10 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
     char cmd_hdata[64], str_signal[128];
 
     /* make C compiler happy */
+    (void) data;
     (void) type_data;
 
-    ptr_client = (struct t_relay_client *)data;
+    ptr_client = (struct t_relay_client *)pointer;
     if (!ptr_client || !relay_client_valid (ptr_client))
         return WEECHAT_RC_OK;
 
@@ -486,7 +529,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,short_name,"
                                              "nicklist,title,local_variables,"
@@ -502,15 +545,16 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
         if (!ptr_buffer)
             return WEECHAT_RC_OK;
 
-        /* send signal only if sync with flag "buffer" */
+        /* send signal only if sync with flag "buffers" or "buffer" */
         if (relay_weechat_protocol_is_sync (ptr_client, ptr_buffer,
+                                            RELAY_WEECHAT_PROTOCOL_SYNC_BUFFERS |
                                             RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER))
         {
             msg = relay_weechat_msg_new (str_signal);
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,type");
                 relay_weechat_msg_send (ptr_client, msg);
@@ -533,7 +577,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,"
                                              "prev_buffer,next_buffer");
@@ -558,7 +602,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,"
                                              "prev_buffer,next_buffer");
@@ -583,7 +627,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,"
                                              "prev_buffer,next_buffer");
@@ -607,7 +651,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,short_name,"
                                              "local_variables");
@@ -631,7 +675,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name,title");
                 relay_weechat_msg_send (ptr_client, msg);
@@ -639,6 +683,29 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             }
         }
     }
+    else if (strncmp (signal, "buffer_localvar_", 16) == 0)
+    {
+        ptr_buffer = (struct t_gui_buffer *)signal_data;
+        if (!ptr_buffer)
+            return WEECHAT_RC_OK;
+
+        /* send signal only if sync with flag "buffers" or "buffer" */
+        if (relay_weechat_protocol_is_sync (ptr_client, ptr_buffer,
+                                            RELAY_WEECHAT_PROTOCOL_SYNC_BUFFERS |
+                                            RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER))
+        {
+            msg = relay_weechat_msg_new (str_signal);
+            if (msg)
+            {
+                snprintf (cmd_hdata, sizeof (cmd_hdata),
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
+                relay_weechat_msg_add_hdata (msg, cmd_hdata,
+                                             "number,full_name,local_variables");
+                relay_weechat_msg_send (ptr_client, msg);
+                relay_weechat_msg_free (msg);
+            }
+        }
+    }
     else if (strcmp (signal, "buffer_cleared") == 0)
     {
         ptr_buffer = (struct t_gui_buffer *)signal_data;
@@ -653,7 +720,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name");
                 relay_weechat_msg_send (ptr_client, msg);
@@ -661,28 +728,6 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             }
         }
     }
-    else if (strncmp (signal, "buffer_localvar_", 16) == 0)
-    {
-        ptr_buffer = (struct t_gui_buffer *)signal_data;
-        if (!ptr_buffer)
-            return WEECHAT_RC_OK;
-
-        /* send signal only if sync with flag "buffer" */
-        if (relay_weechat_protocol_is_sync (ptr_client, ptr_buffer,
-                                            RELAY_WEECHAT_PROTOCOL_SYNC_BUFFER))
-        {
-            msg = relay_weechat_msg_new (str_signal);
-            if (msg)
-            {
-                snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
-                relay_weechat_msg_add_hdata (msg, cmd_hdata,
-                                             "number,full_name,local_variables");
-                relay_weechat_msg_send (ptr_client, msg);
-                relay_weechat_msg_free (msg);
-            }
-        }
-    }
     else if (strcmp (signal, "buffer_line_added") == 0)
     {
         ptr_line = (struct t_gui_line *)signal_data;
@@ -715,7 +760,7 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
                           "line_data:0x%lx",
-                          (long unsigned int)ptr_line_data);
+                          (unsigned long)ptr_line_data);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "buffer,date,date_printed,"
                                              "displayed,highlight,tags_array,"
@@ -740,15 +785,21 @@ relay_weechat_protocol_signal_buffer_cb (void *data, const char *signal,
             if (msg)
             {
                 snprintf (cmd_hdata, sizeof (cmd_hdata),
-                          "buffer:0x%lx", (long unsigned int)ptr_buffer);
-                weechat_hashtable_remove (RELAY_WEECHAT_DATA(ptr_client, buffers_nicklist),
-                                          ptr_buffer);
+                          "buffer:0x%lx", (unsigned long)ptr_buffer);
                 relay_weechat_msg_add_hdata (msg, cmd_hdata,
                                              "number,full_name");
                 relay_weechat_msg_send (ptr_client, msg);
                 relay_weechat_msg_free (msg);
             }
         }
+
+        /* remove buffer from hashtables */
+        weechat_hashtable_remove (
+            RELAY_WEECHAT_DATA(ptr_client, buffers_sync),
+            weechat_buffer_get_string (ptr_buffer, "full_name"));
+        weechat_hashtable_remove (
+            RELAY_WEECHAT_DATA(ptr_client, buffers_nicklist),
+            ptr_buffer);
     }
 
     return WEECHAT_RC_OK;
@@ -813,14 +864,16 @@ relay_weechat_protocol_nicklist_map_cb (void *data,
  */
 
 int
-relay_weechat_protocol_timer_nicklist_cb (void *data, int remaining_calls)
+relay_weechat_protocol_timer_nicklist_cb (const void *pointer, void *data,
+                                          int remaining_calls)
 {
     struct t_relay_client *ptr_client;
 
     /* make C compiler happy */
+    (void) data;
     (void) remaining_calls;
 
-    ptr_client = (struct t_relay_client *)data;
+    ptr_client = (struct t_relay_client *)pointer;
     if (!ptr_client || !relay_client_valid (ptr_client))
         return WEECHAT_RC_OK;
 
@@ -840,7 +893,8 @@ relay_weechat_protocol_timer_nicklist_cb (void *data, int remaining_calls)
  */
 
 int
-relay_weechat_protocol_hsignal_nicklist_cb (void *data, const char *signal,
+relay_weechat_protocol_hsignal_nicklist_cb (const void *pointer, void *data,
+                                            const char *signal,
                                             struct t_hashtable *hashtable)
 {
     struct t_relay_client *ptr_client;
@@ -850,7 +904,10 @@ relay_weechat_protocol_hsignal_nicklist_cb (void *data, const char *signal,
     struct t_relay_weechat_nicklist *ptr_nicklist;
     char diff;
 
-    ptr_client = (struct t_relay_client *)data;
+    /* make C compiler happy */
+    (void) data;
+
+    ptr_client = (struct t_relay_client *)pointer;
     if (!ptr_client || !relay_client_valid (ptr_client))
         return WEECHAT_RC_OK;
 
@@ -933,7 +990,8 @@ relay_weechat_protocol_hsignal_nicklist_cb (void *data, const char *signal,
  */
 
 int
-relay_weechat_protocol_signal_upgrade_cb (void *data, const char *signal,
+relay_weechat_protocol_signal_upgrade_cb (const void *pointer, void *data,
+                                          const char *signal,
                                           const char *type_data,
                                           void *signal_data)
 {
@@ -942,10 +1000,11 @@ relay_weechat_protocol_signal_upgrade_cb (void *data, const char *signal,
     char str_signal[128];
 
     /* make C compiler happy */
+    (void) data;
     (void) type_data;
     (void) signal_data;
 
-    ptr_client = (struct t_relay_client *)data;
+    ptr_client = (struct t_relay_client *)pointer;
     if (!ptr_client || !relay_client_valid (ptr_client))
         return WEECHAT_RC_OK;
 
@@ -981,10 +1040,10 @@ relay_weechat_protocol_signal_upgrade_cb (void *data, const char *signal,
 
 RELAY_WEECHAT_PROTOCOL_CALLBACK(sync)
 {
-    char **buffers, **flags, *full_name;
+    char **buffers, **flags;
+    const char *ptr_full_name;
     int num_buffers, num_flags, i, add_flags, mask, *ptr_old_flags, new_flags;
-    int rc;
-    long unsigned int value;
+    struct t_gui_buffer *ptr_buffer;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
@@ -1010,37 +1069,36 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(sync)
         {
             for (i = 0; i < num_buffers; i++)
             {
-                full_name = NULL;
+                ptr_full_name = NULL;
                 mask = RELAY_WEECHAT_PROTOCOL_SYNC_FOR_BUFFER;
-                if (strncmp (buffers[i], "0x", 2) == 0)
+
+                if (strcmp (buffers[i], "*") == 0)
                 {
-                    rc = sscanf (buffers[i], "%lx", &value);
-                    if ((rc != EOF) && (rc != 0))
-                    {
-                        full_name = strdup (weechat_buffer_get_string ((struct t_gui_buffer *)value,
-                                                                       "full_name"));
-                    }
+                    ptr_full_name = buffers[i];
+                    mask = RELAY_WEECHAT_PROTOCOL_SYNC_ALL;
                 }
                 else
                 {
-                    full_name = strdup (buffers[i]);
-                    if (strcmp (buffers[i], "*") == 0)
-                        mask = RELAY_WEECHAT_PROTOCOL_SYNC_ALL;
+                    ptr_buffer = relay_weechat_protocol_get_buffer (buffers[i]);
+                    if (ptr_buffer)
+                    {
+                        ptr_full_name = weechat_buffer_get_string (ptr_buffer,
+                                                                   "full_name");
+                    }
                 }
 
-                if (full_name)
+                if (ptr_full_name)
                 {
                     ptr_old_flags = weechat_hashtable_get (RELAY_WEECHAT_DATA(client, buffers_sync),
-                                                           full_name);
+                                                           ptr_full_name);
                     new_flags = ((ptr_old_flags) ? *ptr_old_flags : 0);
                     new_flags |= (add_flags & mask);
                     if (new_flags)
                     {
                         weechat_hashtable_set (RELAY_WEECHAT_DATA(client, buffers_sync),
-                                               full_name,
+                                               ptr_full_name,
                                                &new_flags);
                     }
-                    free (full_name);
                 }
             }
         }
@@ -1061,10 +1119,10 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(sync)
 
 RELAY_WEECHAT_PROTOCOL_CALLBACK(desync)
 {
-    char **buffers, **flags, *full_name;
+    char **buffers, **flags;
+    const char *ptr_full_name;
     int num_buffers, num_flags, i, sub_flags, mask, *ptr_old_flags, new_flags;
-    int rc;
-    long unsigned int value;
+    struct t_gui_buffer *ptr_buffer;
 
     RELAY_WEECHAT_PROTOCOL_MIN_ARGS(0);
 
@@ -1090,42 +1148,41 @@ RELAY_WEECHAT_PROTOCOL_CALLBACK(desync)
         {
             for (i = 0; i < num_buffers; i++)
             {
-                full_name = NULL;
+                ptr_full_name = NULL;
                 mask = RELAY_WEECHAT_PROTOCOL_SYNC_FOR_BUFFER;
-                if (strncmp (buffers[i], "0x", 2) == 0)
+
+                if (strcmp (buffers[i], "*") == 0)
                 {
-                    rc = sscanf (buffers[i], "%lx", &value);
-                    if ((rc != EOF) && (rc != 0))
-                    {
-                        full_name = strdup (weechat_buffer_get_string ((struct t_gui_buffer *)value,
-                                                                       "full_name"));
-                    }
+                    ptr_full_name = buffers[i];
+                    mask = RELAY_WEECHAT_PROTOCOL_SYNC_ALL;
                 }
                 else
                 {
-                    full_name = strdup (buffers[i]);
-                    if (strcmp (buffers[i], "*") == 0)
-                        mask = RELAY_WEECHAT_PROTOCOL_SYNC_ALL;
+                    ptr_buffer = relay_weechat_protocol_get_buffer (buffers[i]);
+                    if (ptr_buffer)
+                    {
+                        ptr_full_name = weechat_buffer_get_string (ptr_buffer,
+                                                                   "full_name");
+                    }
                 }
 
-                if (full_name)
+                if (ptr_full_name)
                 {
                     ptr_old_flags = weechat_hashtable_get (RELAY_WEECHAT_DATA(client, buffers_sync),
-                                                           full_name);
+                                                           ptr_full_name);
                     new_flags = ((ptr_old_flags) ? *ptr_old_flags : 0);
                     new_flags &= ~(sub_flags & mask);
                     if (new_flags)
                     {
                         weechat_hashtable_set (RELAY_WEECHAT_DATA(client, buffers_sync),
-                                               full_name,
+                                               ptr_full_name,
                                                &new_flags);
                     }
                     else
                     {
                         weechat_hashtable_remove (RELAY_WEECHAT_DATA(client, buffers_sync),
-                                                  full_name);
+                                                  ptr_full_name);
                     }
-                    free (full_name);
                 }
             }
         }
@@ -1349,7 +1406,7 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
             pos++;
         }
         argv = weechat_string_split (pos, " ", 0, 0, &argc);
-        argv_eol = weechat_string_split (pos, " ", 1, 0, NULL);
+        argv_eol = weechat_string_split (pos, " ", 2, 0, NULL);
     }
 
     for (i = 0; protocol_cb[i].name; i++)
@@ -1357,14 +1414,15 @@ relay_weechat_protocol_recv (struct t_relay_client *client, const char *data)
         if (strcmp (protocol_cb[i].name, command) == 0)
         {
             if ((strcmp (protocol_cb[i].name, "init") != 0)
-                && (!RELAY_WEECHAT_DATA(client, password_ok)))
+                && (!RELAY_WEECHAT_DATA(client, password_ok)
+                    || !RELAY_WEECHAT_DATA(client, totp_ok)))
             {
                 /*
-                 * command is not "init" and password is not set?
+                 * command is not "init" and password or totp are not set?
                  * then close connection!
                  */
                 relay_client_set_status (client,
-                                         RELAY_STATUS_DISCONNECTED);
+                                         RELAY_STATUS_AUTH_FAILED);
             }
             else
             {
