@@ -1,7 +1,7 @@
 /*
  * script-config.c - script configuration options (file script.conf)
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -398,6 +398,7 @@ script_config_init ()
     if (!ptr_section)
     {
         weechat_config_free (script_config_file);
+        script_config_file = NULL;
         return 0;
     }
 
@@ -454,7 +455,7 @@ script_config_init ()
            "u=date updated; char \"-\" can be used before identifier to reverse "
            "order; example: \"i,u\": installed scripts first, sorted by update "
            "date"),
-        NULL, 0, 0, "p,n", NULL, 0,
+        NULL, 0, 0, "i,p,n", NULL, 0,
         NULL, NULL, NULL,
         &script_config_reload_scripts_cb, NULL, NULL,
         NULL, NULL, NULL);
@@ -489,6 +490,7 @@ script_config_init ()
     if (!ptr_section)
     {
         weechat_config_free (script_config_file);
+        script_config_file = NULL;
         return 0;
     }
 
@@ -712,6 +714,7 @@ script_config_init ()
     if (!ptr_section)
     {
         weechat_config_free (script_config_file);
+        script_config_file = NULL;
         return 0;
     }
 
