@@ -1,7 +1,7 @@
 /*
  * trigger-config.c - trigger configuration options (file trigger.conf)
  *
- * Copyright (C) 2014-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -42,6 +42,7 @@ struct t_config_option *trigger_config_color_flag_command;
 struct t_config_option *trigger_config_color_flag_conditions;
 struct t_config_option *trigger_config_color_flag_regex;
 struct t_config_option *trigger_config_color_flag_return_code;
+struct t_config_option *trigger_config_color_flag_post_action;
 struct t_config_option *trigger_config_color_regex;
 struct t_config_option *trigger_config_color_replace;
 struct t_config_option *trigger_config_color_trigger;
@@ -49,22 +50,38 @@ struct t_config_option *trigger_config_color_trigger_disabled;
 
 char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
 {
-    /* beep on highlight/private message */
+    /*
+     * beep on highlight/private message, on following conditions:
+     *   - message is displayed (not filtered)
+     *     AND:
+     *       - message is a highlight
+     *         OR:
+     *       - message is a message in a private buffer
+     */
     { "beep", "on",
       "print",
       "",
       "${tg_displayed} && (${tg_highlight} || ${tg_msg_pv})",
       "",
       "/print -beep",
-      "ok" },
-    /* hide passwords in commands */
+      "ok",
+      "" },
+    /*
+     * hide passwords in commands:
+     *   - /msg nickserv id|identify|ghost|release|regain|recover
+     *   - /oper
+     *   - /quote pass
+     *   - /set *password*
+     *   - /secure passphrase|decrypt|set
+     */
     { "cmd_pass", "on",
       "modifier",
       "5000|input_text_display;5000|history_add;5000|irc_command_auth",
       "",
       "==^("
-      "(/(msg|quote) +nickserv "
-      "+(id|identify|register|ghost +[^ ]+|release +[^ ]+|regain +[^ ]+) +)|"
+      "(/(msg|m|quote) +(-server +[^ ]+ +)?nickserv "
+      "+(id|identify|set +password|ghost +[^ ]+|release +[^ ]+|regain +[^ ]+|"
+      "recover +[^ ]+) +)|"
       "/oper +[^ ]+ +|"
       "/quote +pass +|"
       "/set +[^ ]*password[^ ]* +|"
@@ -72,17 +89,40 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
       "(.*)"
       "==${re:1}${hide:*,${re:+}}",
       "",
+      "",
       "" },
-    /* hide password in IRC auth message displayed */
+    /*
+     * hide passwords in commands:
+     *   - /msg nickserv register
+     */
+    { "cmd_pass_register", "on",
+      "modifier",
+      "5000|input_text_display;5000|history_add;5000|irc_command_auth",
+      "",
+      "==^(/(msg|m|quote) +nickserv +register +)([^ ]+)(.*)"
+      "==${re:1}${hide:*,${re:3}}${re:4}",
+      "",
+      "",
+      "" },
+    /*
+     * hide password in IRC auth message displayed
+     * (message received from server after the user issued the command)
+     */
     { "msg_auth", "on",
       "modifier",
       "5000|irc_message_auth",
       "",
-      "==^(.*(id|identify|register|ghost +[^ ]+|release +[^ ]+) +)(.*)"
+      "==^(.*(id|identify|set +password|register|ghost +[^ ]+|release +[^ ]+|"
+      "regain +[^ ]+|recover +[^ ]+) +)(.*)"
       "==${re:1}${hide:*,${re:+}}",
       "",
+      "",
       "" },
-    /* hide server password in commands /server and /connect */
+    /*
+     * hide server password in commands:
+     *   - /server
+     *   - /connect
+     */
     { "server_pass", "on",
       "modifier",
       "5000|input_text_display;5000|history_add",
@@ -90,8 +130,9 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
       "==^(/(server|connect) .*-(sasl_)?password=)([^ ]+)(.*)"
       "==${re:1}${hide:*,${re:4}}${re:5}"
       "",
+      "",
       "" },
-    { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
+    { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
 };
 
 
@@ -100,9 +141,11 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
  */
 
 void
-trigger_config_change_enabled (void *data, struct t_config_option *option)
+trigger_config_change_enabled (const void *pointer, void *data,
+                               struct t_config_option *option)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     trigger_enabled = weechat_config_boolean (option);
@@ -113,12 +156,13 @@ trigger_config_change_enabled (void *data, struct t_config_option *option)
  */
 
 void
-trigger_config_change_trigger_enabled (void *data,
+trigger_config_change_trigger_enabled (const void *pointer, void *data,
                                        struct t_config_option *option)
 {
     struct t_trigger *ptr_trigger;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     ptr_trigger = trigger_search_with_option (option);
@@ -136,11 +180,13 @@ trigger_config_change_trigger_enabled (void *data,
  */
 
 void
-trigger_config_change_trigger_hook (void *data, struct t_config_option *option)
+trigger_config_change_trigger_hook (const void *pointer, void *data,
+                                    struct t_config_option *option)
 {
     struct t_trigger *ptr_trigger;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     ptr_trigger = trigger_search_with_option (option);
@@ -156,12 +202,13 @@ trigger_config_change_trigger_hook (void *data, struct t_config_option *option)
  */
 
 void
-trigger_config_change_trigger_arguments (void *data,
+trigger_config_change_trigger_arguments (const void *pointer, void *data,
                                          struct t_config_option *option)
 {
     struct t_trigger *ptr_trigger;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     ptr_trigger = trigger_search_with_option (option);
@@ -177,11 +224,13 @@ trigger_config_change_trigger_arguments (void *data,
  */
 
 void
-trigger_config_change_trigger_regex (void *data, struct t_config_option *option)
+trigger_config_change_trigger_regex (const void *pointer, void *data,
+                                     struct t_config_option *option)
 {
     struct t_trigger *ptr_trigger;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     ptr_trigger = trigger_search_with_option (option);
@@ -221,12 +270,13 @@ trigger_config_change_trigger_regex (void *data, struct t_config_option *option)
  */
 
 void
-trigger_config_change_trigger_command (void *data,
+trigger_config_change_trigger_command (const void *pointer, void *data,
                                        struct t_config_option *option)
 {
     struct t_trigger *ptr_trigger;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     ptr_trigger = trigger_search_with_option (option);
@@ -271,8 +321,10 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                 option_name, "boolean",
                 N_("if disabled, the hooks are removed from trigger, so it is "
                    "not called any more"),
-                NULL, 0, 0, value, NULL, 0, NULL, NULL,
-                &trigger_config_change_trigger_enabled, NULL, NULL, NULL);
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &trigger_config_change_trigger_enabled, NULL, NULL,
+                NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_HOOK:
             ptr_option = weechat_config_new_option (
@@ -280,8 +332,10 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                 option_name, "integer",
                 N_("type of hook used"),
                 trigger_hook_option_values,
-                0, 0, value, NULL, 0, NULL, NULL,
-                &trigger_config_change_trigger_hook, NULL, NULL, NULL);
+                0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &trigger_config_change_trigger_hook, NULL, NULL,
+                NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_ARGUMENTS:
             ptr_option = weechat_config_new_option (
@@ -289,8 +343,10 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                 option_name, "string",
                 N_("arguments for the hook (depend on the hook type, see /help "
                    "trigger)"),
-                NULL, 0, 0, value, NULL, 0, NULL, NULL,
-                &trigger_config_change_trigger_arguments, NULL, NULL, NULL);
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &trigger_config_change_trigger_arguments, NULL, NULL,
+                NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_CONDITIONS:
             ptr_option = weechat_config_new_option (
@@ -300,7 +356,7 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                    "hook callback) (note: content is evaluated when trigger is "
                    "run, see /help eval)"),
                 NULL, 0, 0, value, NULL, 0,
-                NULL, NULL, NULL, NULL, NULL, NULL);
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_REGEX:
             ptr_option = weechat_config_new_option (
@@ -319,8 +375,10 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                    "replace: ${re:0} to ${re:99}, ${re:+} for last match and "
                    "${hide:c,${re:N}} to replace all chars of group N by "
                    "char 'c'"),
-                NULL, 0, 0, value, NULL, 0, NULL, NULL,
-                &trigger_config_change_trigger_regex, NULL, NULL, NULL);
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &trigger_config_change_trigger_regex, NULL, NULL,
+                NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_COMMAND:
             ptr_option = weechat_config_new_option (
@@ -328,8 +386,10 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                 option_name, "string",
                 N_("command(s) to run if conditions are OK, after regex "
                    "replacements (many commands can be separated by semicolons)"),
-                NULL, 0, 0, value, NULL, 0, NULL, NULL,
-                &trigger_config_change_trigger_command, NULL, NULL, NULL);
+                NULL, 0, 0, value, NULL, 0,
+                NULL, NULL, NULL,
+                &trigger_config_change_trigger_command, NULL, NULL,
+                NULL, NULL, NULL);
             break;
         case TRIGGER_OPTION_RETURN_CODE:
             ptr_option = weechat_config_new_option (
@@ -338,7 +398,15 @@ trigger_config_create_trigger_option (const char *trigger_name, int index_option
                 N_("return code for hook callback (see plugin API reference to "
                    "know where ok_eat/error can be used efficiently)"),
                 "ok|ok_eat|error", 0, 0, value, NULL, 0,
-                NULL, NULL, NULL, NULL, NULL, NULL);
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+            break;
+        case TRIGGER_OPTION_POST_ACTION:
+            ptr_option = weechat_config_new_option (
+                trigger_config_file, trigger_config_section_trigger,
+                option_name, "integer",
+                N_("action to take on the trigger after execution"),
+                "none|disable|delete", 0, 0, value, NULL, 0,
+                NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
             break;
         case TRIGGER_NUM_OPTIONS:
             break;
@@ -432,7 +500,8 @@ trigger_config_use_temp_triggers ()
  */
 
 int
-trigger_config_trigger_read_cb (void *data, struct t_config_file *config_file,
+trigger_config_trigger_read_cb (const void *pointer, void *data,
+                                struct t_config_file *config_file,
                                 struct t_config_section *section,
                                 const char *option_name, const char *value)
 {
@@ -441,6 +510,7 @@ trigger_config_trigger_read_cb (void *data, struct t_config_file *config_file,
     int index_option;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) config_file;
     (void) section;
@@ -502,7 +572,7 @@ trigger_config_trigger_read_cb (void *data, struct t_config_file *config_file,
  */
 
 int
-trigger_config_trigger_write_default_cb (void *data,
+trigger_config_trigger_write_default_cb (const void *pointer, void *data,
                                          struct t_config_file *config_file,
                                          const char *section_name)
 {
@@ -510,6 +580,7 @@ trigger_config_trigger_write_default_cb (void *data,
     char option_name[512];
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     if (!weechat_config_write_line (config_file, section_name, NULL))
@@ -543,11 +614,13 @@ trigger_config_trigger_write_default_cb (void *data,
  */
 
 int
-trigger_config_reload_cb (void *data, struct t_config_file *config_file)
+trigger_config_reload_cb (const void *pointer, void *data,
+                          struct t_config_file *config_file)
 {
     int rc;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     trigger_free_all ();
@@ -572,20 +645,23 @@ trigger_config_init ()
 {
     struct t_config_section *ptr_section;
 
-    trigger_config_file = weechat_config_new (TRIGGER_CONFIG_NAME,
-                                              &trigger_config_reload_cb, NULL);
+    trigger_config_file = weechat_config_new (
+        TRIGGER_CONFIG_NAME, &trigger_config_reload_cb, NULL, NULL);
     if (!trigger_config_file)
         return 0;
 
     /* look */
     ptr_section = weechat_config_new_section (trigger_config_file, "look",
                                               0, 0,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL);
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (trigger_config_file);
+        trigger_config_file = NULL;
         return 0;
     }
 
@@ -593,24 +669,29 @@ trigger_config_init ()
         trigger_config_file, ptr_section,
         "enabled", "boolean",
         N_("enable trigger support"),
-        NULL, 0, 0, "on", NULL, 0, NULL, NULL,
-        &trigger_config_change_enabled, NULL, NULL, NULL);
+        NULL, 0, 0, "on", NULL, 0,
+        NULL, NULL, NULL,
+        &trigger_config_change_enabled, NULL, NULL,
+        NULL, NULL, NULL);
     trigger_config_look_monitor_strip_colors = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "monitor_strip_colors", "boolean",
         N_("strip colors in hashtable values displayed on monitor buffer"),
-        NULL, 0, 0, "off", NULL, 0, NULL, NULL,
-        NULL, NULL, NULL, NULL);
+        NULL, 0, 0, "off", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
     /* color */
     ptr_section = weechat_config_new_section (trigger_config_file, "color",
                                               0, 0,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL);
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (trigger_config_file);
+        trigger_config_file = NULL;
         return 0;
     }
 
@@ -619,62 +700,70 @@ trigger_config_init ()
         "flag_command", "color",
         N_("text color for command flag (in /trigger list)"),
         NULL, 0, 0, "lightgreen", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_flag_conditions = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "flag_conditions", "color",
         N_("text color for conditions flag (in /trigger list)"),
         NULL, 0, 0, "yellow", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_flag_regex = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "flag_regex", "color",
         N_("text color for regex flag (in /trigger list)"),
         NULL, 0, 0, "lightcyan", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_flag_return_code = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "flag_return_code", "color",
         N_("text color for return code flag (in /trigger list)"),
         NULL, 0, 0, "lightmagenta", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    trigger_config_color_flag_post_action = weechat_config_new_option (
+        trigger_config_file, ptr_section,
+        "flag_post_action", "color",
+        N_("text color for post action flag (in /trigger list)"),
+        NULL, 0, 0, "lightblue", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_regex = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "regex", "color",
         N_("text color for regular expressions"),
         NULL, 0, 0, "white", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_replace = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "replace", "color",
         N_("text color for replacement text (for regular expressions)"),
         NULL, 0, 0, "cyan", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_trigger = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "trigger", "color",
         N_("text color for trigger name"),
         NULL, 0, 0, "green", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
     trigger_config_color_trigger_disabled = weechat_config_new_option (
         trigger_config_file, ptr_section,
         "trigger_disabled", "color",
         N_("text color for disabled trigger name"),
         NULL, 0, 0, "red", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
     /* trigger */
-    ptr_section = weechat_config_new_section (trigger_config_file,
-                                              TRIGGER_CONFIG_SECTION_TRIGGER,
-                                              0, 0,
-                                              &trigger_config_trigger_read_cb, NULL,
-                                              NULL, NULL,
-                                              &trigger_config_trigger_write_default_cb, NULL,
-                                              NULL, NULL,
-                                              NULL, NULL);
+    ptr_section = weechat_config_new_section (
+        trigger_config_file,
+        TRIGGER_CONFIG_SECTION_TRIGGER,
+        0, 0,
+        &trigger_config_trigger_read_cb, NULL, NULL,
+        NULL, NULL, NULL,
+        &trigger_config_trigger_write_default_cb, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (trigger_config_file);
+        trigger_config_file = NULL;
         return 0;
     }
 
