@@ -1,7 +1,7 @@
 /*
  * tests.cpp - run WeeChat tests
  *
- * Copyright (C) 2014-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,13 +16,14 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <iostream>
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <dlfcn.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -52,18 +53,28 @@ extern "C"
 
 #define WEECHAT_TESTS_HOME "./tmp_weechat_test"
 
+/* lib with tests on plugins when autotools is used to compile */
+#define WEECHAT_TESTS_PLUGINS_LIB_DEFAULT                       \
+    "./tests/.libs/lib_weechat_unit_tests_plugins.so.0.0.0"
+
 /* import tests from libs */
+/* core */
 IMPORT_TEST_GROUP(Plugins);
-IMPORT_TEST_GROUP(Arraylist);
-IMPORT_TEST_GROUP(Eval);
-IMPORT_TEST_GROUP(Hashtable);
-IMPORT_TEST_GROUP(Hdata);
-IMPORT_TEST_GROUP(Infolist);
-IMPORT_TEST_GROUP(List);
-IMPORT_TEST_GROUP(String);
-IMPORT_TEST_GROUP(Url);
-IMPORT_TEST_GROUP(Utf8);
-IMPORT_TEST_GROUP(Util);
+IMPORT_TEST_GROUP(CoreArraylist);
+IMPORT_TEST_GROUP(CoreEval);
+IMPORT_TEST_GROUP(CoreHashtable);
+IMPORT_TEST_GROUP(CoreHdata);
+IMPORT_TEST_GROUP(CoreHook);
+IMPORT_TEST_GROUP(CoreInfolist);
+IMPORT_TEST_GROUP(CoreList);
+IMPORT_TEST_GROUP(CoreSecure);
+IMPORT_TEST_GROUP(CoreString);
+IMPORT_TEST_GROUP(CoreUrl);
+IMPORT_TEST_GROUP(CoreUtf8);
+IMPORT_TEST_GROUP(CoreUtil);
+/* GUI */
+IMPORT_TEST_GROUP(GuiLine);
+/* scripts */
 IMPORT_TEST_GROUP(Scripts);
 
 struct t_gui_buffer *ptr_core_buffer = NULL;
@@ -101,6 +112,13 @@ test_print_cb (const void *pointer, void *data, struct t_gui_buffer *buffer,
     (void) displayed;
     (void) highlight;
 
+    /* keep only messages displayed on core buffer */
+    if (strcmp (gui_buffer_get_string (buffer, "full_name"),
+                "core.weechat") != 0)
+    {
+        return WEECHAT_RC_OK;
+    }
+
     printf ("%s%s%s\n",  /* with color: "\33[34m%s%s%s\33[0m\n" */
             (prefix && prefix[0]) ? prefix : "",
             (prefix && prefix[0]) ? " " : "",
@@ -155,7 +173,10 @@ int
 main (int argc, char *argv[])
 {
     int rc, length, weechat_argc;
-    char *weechat_tests_args, *args, **weechat_argv;
+    char *weechat_tests_args, *args, **weechat_argv, *tests_plugins_lib;
+    const char *tests_plugins_lib_default = WEECHAT_TESTS_PLUGINS_LIB_DEFAULT;
+    const char *ptr_path;
+    void *handle;
 
     /* setup environment: English language, no specific timezone */
     setenv ("LC_ALL", LOCALE_TESTS, 1);
@@ -212,7 +233,20 @@ main (int argc, char *argv[])
                          "Auto-loading plugins from path in "
                          "environment variable WEECHAT_EXTRA_LIBDIR (\"%s\")",
                          getenv ("WEECHAT_EXTRA_LIBDIR"));
-        plugin_auto_load (0, NULL, 0, 1, 0);
+        plugin_auto_load (NULL, 0, 1, 0, 0, NULL);
+    }
+
+    /* load plugins tests */
+    tests_plugins_lib = getenv ("WEECHAT_TESTS_PLUGINS_LIB");
+    ptr_path = (tests_plugins_lib && tests_plugins_lib[0]) ?
+        tests_plugins_lib : tests_plugins_lib_default;
+    printf ("Loading tests on plugins: \"%s\"\n", ptr_path);
+    handle = dlopen (ptr_path, RTLD_GLOBAL | RTLD_NOW);
+    if (!handle)
+    {
+        fprintf (stderr, "ERROR: unable to load tests on plugins: %s\n",
+                 dlerror ());
+        return 1;
     }
 
     /* display WeeChat version and directories */
@@ -237,5 +271,7 @@ main (int argc, char *argv[])
             (rc == 0) ? 32 : 31,  /* 32 = green (OK), 31 = red (error) */
             (rc == 0) ? "OK" : "ERROR");
 
+    dlclose (handle);
+
     return rc;
 }
